@@ -33,34 +33,31 @@
 #include <gtkmm/main.h>
 #include <string>
 #include <list>
+#include <set>
 
 #include <ETL/smart_ptr>
 
 #include <synfig/distance.h>
 #include <synfig/string.h>
 #include <synfig/time.h>
-
+#include <synfig/color.h>
+#include <synfig/layers/layer_bitmap.h>
 #include <synfig/filecontainerzip.h>
 
 #include <gtkmm/uimanager.h>
-
 #include <synfigapp/instance.h>
-#include <synfigapp/pluginmanager.h>
+#include <synfigapp/canvasinterface.h>
+#include "pluginmanager.h"
 #include "iconcontroller.h"
 #include "mainwindow.h"
 
 /* === M A C R O S ========================================================= */
 
-#define MISC_DIR_PREFERENCE			"misc_dir"
-#define ANIMATION_DIR_PREFERENCE	"animation_dir"
-#define IMAGE_DIR_PREFERENCE		"image_dir"
-#define SKETCH_DIR_PREFERENCE		"sketch_dir"
-#define RENDER_DIR_PREFERENCE		"render_dir"
-
-// uncomment define SINGLE_THREADED to use a single thread, and hopefully get more stability (dooglus) - changed to be default if using windows (Nov 2009 pixelgeek)
-#ifdef _WIN32
-#define SINGLE_THREADED
-#endif
+#define MISC_DIR_PREFERENCE       "misc_dir"
+#define ANIMATION_DIR_PREFERENCE  "animation_dir"
+#define IMAGE_DIR_PREFERENCE      "image_dir"
+#define SKETCH_DIR_PREFERENCE     "sketch_dir"
+#define RENDER_DIR_PREFERENCE     "render_dir"
 
 /* === T Y P E D E F S ===================================================== */
 
@@ -72,6 +69,11 @@ namespace Gtk
 	class ActionGroup;
 };
 
+namespace synfig
+{
+	class SoundProcessor;
+};
+
 namespace synfigapp
 {
 	class UIInterface;
@@ -93,6 +95,7 @@ class Dialog_Gradient;
 class Dialog_Input;
 class Dialog_Color;
 class Dialog_ToolOptions;
+class VectorizerSettings;
 class DeviceTracker;
 class AutoRecover;
 
@@ -110,6 +113,8 @@ class Dock_Children;
 class Dock_Info;
 class Dock_Navigator;
 class Dock_LayerGroups;
+class Dock_SoundWave;
+
 class IPC;
 
 class Module;
@@ -117,6 +122,8 @@ class Module;
 class StateManager;
 class IconController;
 
+class WorkspaceHandler;
+
 class App : public Gtk::Main, private IconController
 {
 	friend class Preferences;
@@ -176,6 +183,8 @@ private:
 
 //	static std::list< etl::handle< Module > > module_list_;
 
+	static WorkspaceHandler *workspaces;
+
 	/*
  -- ** -- P U B L I C   D A T A -----------------------------------------------
 	*/
@@ -194,11 +203,9 @@ public:
 	static Dialog_Color* dialog_color;
 //	static Dialog_Palette* dialog_palette;
 	static Dialog_ToolOptions *dialog_tool_options;
-
+	static VectorizerSettings *vectorizerpopup;
 	static synfig::Distance::System distance_system;
 
-	static synfig::Gamma gamma;
-
 	static About *about;
 	static MainWindow *main_window;
 	static Dock_Toolbox *dock_toolbox;
@@ -207,20 +214,14 @@ public:
 
 	static bool shutdown_in_progress;
 
-	static bool use_colorspace_gamma;
-
-#ifdef SINGLE_THREADED
-	static bool single_threaded;
-#endif
-
 	static bool restrict_radius_ducks;
 	static bool resize_imported_images;
 	static bool enable_experimental_features;
 	static bool use_dark_theme;
 	static bool show_file_toolbar;
 
-	static synfigapp::PluginManager plugin_manager;
-
+	static PluginManager plugin_manager;
+	static synfig::String image_editor_path;
 	static std::set< synfig::String > brushes_path;
 	static synfig::String custom_filename_prefix;
 	static int preferred_x_size;
@@ -234,6 +235,18 @@ public:
 	static bool enable_mainwin_menubar;
 	static synfig::String ui_language;
 	static long ui_handle_tooltip_flag;
+	static synfig::String default_background_layer_type;
+	static synfig::Color  default_background_layer_color;
+	static synfig::String default_background_layer_image;
+	static synfig::Color  preview_background_color;
+
+	//The sound effects that will be used
+	static synfig::SoundProcessor* sound_render_done;
+	static bool use_render_done_sound;
+
+	static Dock_Info* dock_info_; //For Render ProgressBar
+
+	static WorkspaceHandler * get_workspace_handler() {return workspaces;}
 
 	/*
  -- ** -- S I G N A L S -------------------------------------------------------
@@ -264,6 +277,8 @@ public:
 
 	static sigc::signal<void> &signal_recent_files_changed();
 
+	static sigc::signal<void> &signal_custom_workspaces_changed();
+
 	static sigc::signal<
 		void,
 		etl::loose_handle<CanvasView>
@@ -321,11 +336,18 @@ public:
 	static void set_workspace_default();
 	static void set_workspace_compositing();
 	static void set_workspace_animating();
+	static void set_workspace_from_template(const std::string &tpl);
+	static void set_workspace_from_name(const std::string &name);
+	static void load_custom_workspaces();
+	static void save_custom_workspace();
+	static void edit_custom_workspace_list();
 	static void restore_default_settings();
 	static void apply_gtk_settings();
 
 	static const std::list<std::string>& get_recent_files();
 
+	static const std::vector<std::string> get_workspaces();
+
 	static const etl::handle<synfigapp::UIInterface>& get_ui_interface();
 
 
@@ -336,12 +358,22 @@ public:
 
 	static etl::loose_handle<Instance> get_selected_instance() { return selected_instance; }
 	static etl::loose_handle<CanvasView> get_selected_canvas_view() { return selected_canvas_view; }
+	static synfig::Gamma get_selected_canvas_gamma();
 
-	static bool open(std::string filename);
+	static std::string get_temporary_directory();
 
-	static bool open_as(std::string filename,std::string as,synfig::FileContainerZip::file_size_t truncate_storage_size = 0);
+	static synfig::FileSystemTemporary::Handle wrap_into_temporary_filesystem(
+		synfig::FileSystem::Handle canvas_file_system,
+		std::string filename,
+		std::string as,
+		synfig::FileContainerZip::file_size_t truncate_storage_size = 0 );
 
-	static bool open_from_temporary_container_as(std::string container_filename_base,std::string as);
+	static bool open(
+		std::string filename,
+		/* std::string as, */
+		synfig::FileContainerZip::file_size_t truncate_storage_size = 0 );
+
+	static bool open_from_temporary_filesystem(std::string temporary_filename);
 
 	static void new_instance();
 
@@ -381,6 +413,7 @@ public:
 	static bool dialog_save_file_spal(const std::string &title, std::string &filename, std::string preference);
 	static bool dialog_save_file_sketch(const std::string &title, std::string &filename, std::string preference);
 	static bool dialog_save_file_render(const std::string &title, std::string &filename, std::string preference);
+	static bool dialog_open_file_image_sequence(const std::string &title, std::set<synfig::String> &filenames, std::string preference);
 
 	static bool dialog_select_list_item(const std::string &title, const std::string &message, const std::list<std::string> &list, int &item_index);
 
@@ -413,6 +446,11 @@ public:
 			const std::string &button3);
 
 	static void open_uri(const std::string &uri);
+	static void open_img_in_external(const std::string &uri);
+	static void open_vectorizerpopup(const etl::handle<synfig::Layer_Bitmap> my_layer_bitmap,
+	const etl::handle<synfig::Layer> reference_layer);
+
+
 
 	static synfig::String get_config_file(const synfig::String& file);
 	// This will spread the changes made in preferences.
@@ -420,7 +458,8 @@ public:
 	// This fixes bug 1890020
 	static void setup_changed();
 
-	static void process_all_events();
+	static void process_all_events(long unsigned int us = 1);
+	static bool check_python_version( std::string path);
 }; // END of class App
 
 	void delete_widget(Gtk::Widget *widget);
