@@ -35,27 +35,14 @@
 #include <glibmm/thread.h>
 #include <glibmm/dispatcher.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
-#ifdef HAVE_SYS_WAIT_H
-#include <sys/wait.h>
-#endif
-
-#ifdef HAVE_SIGNAL_H
-#include <signal.h>
-#endif
-
 #include <synfig/general.h>
 #include <synfig/context.h>
 #include <ETL/clock>
 
 #include <gui/localization.h>
+#include <docks/dock_info.h>
+
+#include <gui/progresslogger.h>
 
 #endif
 
@@ -116,7 +103,6 @@ public:
 		set_alpha_mode(warm_target->get_alpha_mode());
 		set_threads(warm_target->get_threads());
 		set_clipping(warm_target->get_clipping());
-		set_allow_multithreading(warm_target->get_allow_multithreading());
 		set_rend_desc(&warm_target->rend_desc());
 		set_engine(warm_target->get_engine());
 		alive_flag=true;
@@ -135,24 +121,21 @@ public:
 		alive_flag=false;
 	}
 
-	virtual bool async_render_tile(synfig::RectInt rect, synfig::Context context, synfig::RendDesc tile_desc, synfig::ProgressCallback *cb=NULL)
+	virtual bool async_render_tile(
+		etl::handle<Canvas> canvas,
+		ContextParams context_params,
+		RectInt rect,
+		RendDesc tile_desc,
+		ProgressCallback */*cb*/ )
 	{
 		if(!alive_flag)
 			return false;
 
-#ifdef SINGLE_THREADED
-		if (single_threaded())
-			return sync_render_tile(rect, context, tile_desc, cb);
-#endif
-
-		if (!get_allow_multithreading())
-			return sync_render_tile(rect, context, tile_desc, cb);
-
 		Glib::Thread *thread = Glib::Thread::create(
 			sigc::hide_return(
 				sigc::bind(
 					sigc::mem_fun(*this, &AsyncTarget_Tile::sync_render_tile),
-					rect, context, tile_desc, (synfig::ProgressCallback*)NULL )),
+					canvas, context_params, rect, tile_desc, (synfig::ProgressCallback*)NULL )),
 				true
 			);
 		assert(thread);
@@ -165,11 +148,16 @@ public:
 		return true;
 	}
 
-	bool sync_render_tile(synfig::RectInt rect, synfig::Context context, synfig::RendDesc tile_desc, synfig::ProgressCallback *cb)
+	bool sync_render_tile(
+		etl::handle<Canvas> canvas,
+		ContextParams context_params,
+		RectInt rect,
+		RendDesc tile_desc,
+		ProgressCallback *cb )
 	{
 		if(!alive_flag)
 			return false;
-		bool r = warm_target->async_render_tile(rect, context, tile_desc, cb);
+		bool r = warm_target->async_render_tile(canvas, context_params, rect, tile_desc, cb);
 		if (!r) { Glib::Mutex::Lock lock(mutex); err = true; }
 		return r;
 	}
@@ -273,29 +261,22 @@ public:
 
 	virtual void end_frame()
 	{
-#ifdef SINGLE_THREADED
-		if (!single_threaded())
+		while(alive_flag)
 		{
-#endif
-			while(alive_flag)
+			Glib::Mutex::Lock lock(mutex);
+			Glib::TimeVal end_time;
+
+			end_time.assign_current_time();
+			end_time.add_microseconds(BOREDOM_TIMEOUT);
+
+			if(!tile_queue.empty() && alive_flag)
 			{
-				Glib::Mutex::Lock lock(mutex);
-				Glib::TimeVal end_time;
-
-				end_time.assign_current_time();
-				end_time.add_microseconds(BOREDOM_TIMEOUT);
-
-				if(!tile_queue.empty() && alive_flag)
-				{
-					if(cond_tile_queue_empty.timed_wait(mutex,end_time))
-						break;
-				}
-				else
+				if(cond_tile_queue_empty.timed_wait(mutex,end_time))
 					break;
 			}
-#ifdef SINGLE_THREADED
+			else
+				break;
 		}
-#endif
 		Glib::Mutex::Lock lock(mutex);
 		if(!alive_flag)
 			return;
@@ -454,29 +435,22 @@ public:
 	
 	virtual void end_frame()
 	{
-#ifdef SINGLE_THREADED
-		if (!single_threaded())
+		while(alive_flag)
 		{
-#endif
-			while(alive_flag)
+			Glib::Mutex::Lock lock(mutex);
+			Glib::TimeVal end_time;
+			
+			end_time.assign_current_time();
+			end_time.add_microseconds(BOREDOM_TIMEOUT);
+			
+			if(!tile_queue.empty() && alive_flag)
 			{
-				Glib::Mutex::Lock lock(mutex);
-				Glib::TimeVal end_time;
-				
-				end_time.assign_current_time();
-				end_time.add_microseconds(BOREDOM_TIMEOUT);
-				
-				if(!tile_queue.empty() && alive_flag)
-				{
-					if(cond_tile_queue_empty.timed_wait(mutex,end_time))
-						break;
-				}
-				else
+				if(cond_tile_queue_empty.timed_wait(mutex,end_time))
 					break;
 			}
-#ifdef SINGLE_THREADED
+			else
+				break;
 		}
-#endif
 		Glib::Mutex::Lock lock(mutex);
 		if(!alive_flag)
 			return;
@@ -504,13 +478,15 @@ public:
 	bool ready_next;
 	sigc::connection ready_connection;
 
+	ProgressCallback *cb;
 
 public:
 	AsyncTarget_Scanline(etl::handle<synfig::Target_Scanline> warm_target):
 		warm_target(warm_target),
 		scanline_(),
 		alive_flag(),
-		ready_next()
+		ready_next(),
+		cb(nullptr)
 	{
 		set_avoid_time_sync(warm_target->get_avoid_time_sync());
 		set_canvas(warm_target->get_canvas());
@@ -544,8 +520,9 @@ public:
 		alive_flag=false;
 	}
 
-	virtual bool start_frame(synfig::ProgressCallback */*cb*/=0)
+	virtual bool start_frame(synfig::ProgressCallback *cb)
 	{
+		this->cb = cb;
 		return alive_flag;
 	}
 
@@ -571,24 +548,16 @@ public:
 #endif
 		}
 
-#ifdef SINGLE_THREADED
-		if (single_threaded())
-			signal_progress()();
-		else
-#endif
+		Glib::TimeVal end_time;
+
+		end_time.assign_current_time();
+		end_time.add_microseconds(BOREDOM_TIMEOUT);
+
+		while(alive_flag && !ready_next)
 		{
-
-			Glib::TimeVal end_time;
-
-			end_time.assign_current_time();
-			end_time.add_microseconds(BOREDOM_TIMEOUT);
-
-			while(alive_flag && !ready_next)
-			{
-				Glib::Mutex::Lock lock(mutex);
-				if(cond_frame_queue_empty.timed_wait(mutex, end_time))
-					break;
-			}
+			Glib::Mutex::Lock lock(mutex);
+			if(cond_frame_queue_empty.timed_wait(mutex, end_time))
+				break;
 		}
 	}
 
@@ -609,12 +578,19 @@ public:
 	{
 		Glib::Mutex::Lock lock(mutex);
 		if(alive_flag)
-			alive_flag=warm_target->add_frame(&surface);
-#ifdef SINGLE_THREADED
-		if (!single_threaded())
-#endif
-			cond_frame_queue_empty.signal();
+			alive_flag=warm_target->add_frame(&surface, cb);
+		cond_frame_queue_empty.signal();
 		ready_next=true;
+		
+		int n_total_frames_to_render = warm_target->desc.get_frame_end()        //120
+		                             - warm_target->desc.get_frame_start()      //0
+		                             + 1;                                       //->121
+		int current_rendered_frames_count = warm_target->curr_frame_;
+		
+		float r = (float) current_rendered_frames_count 
+		        / (float) n_total_frames_to_render;
+		
+		App::dock_info_->set_render_progress(r);		
 	}
 };
 
@@ -696,24 +672,16 @@ public:
 #endif
 		}
 		
-#ifdef SINGLE_THREADED
-		if (single_threaded())
-			signal_progress()();
-		else
-#endif
+		Glib::TimeVal end_time;
+		
+		end_time.assign_current_time();
+		end_time.add_microseconds(BOREDOM_TIMEOUT);
+		
+		while(alive_flag && !ready_next)
 		{
-			
-			Glib::TimeVal end_time;
-			
-			end_time.assign_current_time();
-			end_time.add_microseconds(BOREDOM_TIMEOUT);
-			
-			while(alive_flag && !ready_next)
-			{
-				Glib::Mutex::Lock lock(mutex);
-				if(cond_frame_queue_empty.timed_wait(mutex, end_time))
-					break;
-			}
+			Glib::Mutex::Lock lock(mutex);
+			if(cond_frame_queue_empty.timed_wait(mutex, end_time))
+				break;
 		}
 		return true;
 	}
@@ -723,11 +691,18 @@ public:
 		Glib::Mutex::Lock lock(mutex);
 		if(alive_flag)
 			alive_flag=warm_target->put_surface(surface, callback);
-#ifdef SINGLE_THREADED
-		if (!single_threaded())
-#endif
-			cond_frame_queue_empty.signal();
+		cond_frame_queue_empty.signal();
 		ready_next=true;
+		
+		int n_total_frames_to_render = warm_target->desc.get_frame_end()        //120
+		                             - warm_target->desc.get_frame_start()      //0
+		                             + 1;                                       //->121
+		int current_rendered_frames_count = warm_target->curr_frame_;
+		
+		float r = (float) current_rendered_frames_count 
+		        / (float) n_total_frames_to_render;
+		
+		App::dock_info_->set_render_progress(r);
 	}
 
 	virtual bool obtain_surface(cairo_surface_t*& s)
@@ -747,58 +722,48 @@ public:
 /* === M E T H O D S ======================================================= */
 
 AsyncRenderer::AsyncRenderer(etl::handle<synfig::Target> target_,synfig::ProgressCallback *cb):
-	error(false),
-	success(false),
+	status(RENDERING_UNDEFINED),
 	cb(cb),
-#ifdef SINGLE_THREADED
-	updating(false),
-#endif
 	start_clock(0),
 	finish_clock(0),
 	start_time(0, 0),
 	finish_time(0, 0)
 {
 	render_thread=0;
-	if(etl::handle<synfig::Target_Tile>::cast_dynamic(target_))
+	if(auto cast_target = synfig::Target_Tile::Handle::cast_dynamic(target_))
 	{
 		etl::handle<AsyncTarget_Tile> wrap_target(
-			new AsyncTarget_Tile(etl::handle<synfig::Target_Tile>::cast_dynamic(target_))
+			new AsyncTarget_Tile(cast_target)
 		);
 
 		signal_stop_.connect(sigc::mem_fun(*wrap_target,&AsyncTarget_Tile::set_dead));
 
 		target=wrap_target;
 	}
-	else if(etl::handle<synfig::Target_Scanline>::cast_dynamic(target_))
+	else if(auto cast_target = etl::handle<synfig::Target_Scanline>::cast_dynamic(target_))
 	{
 		etl::handle<AsyncTarget_Scanline> wrap_target(
-			new AsyncTarget_Scanline(
-				etl::handle<synfig::Target_Scanline>::cast_dynamic(target_)
-			)
+			new AsyncTarget_Scanline(cast_target)
 		);
 
 		signal_stop_.connect(sigc::mem_fun(*wrap_target,&AsyncTarget_Scanline::set_dead));
 
 		target=wrap_target;
 	}
-	else if(etl::handle<synfig::Target_Cairo_Tile>::cast_dynamic(target_))
+	else if(auto cast_target = etl::handle<synfig::Target_Cairo_Tile>::cast_dynamic(target_))
 	{
 		etl::handle<AsyncTarget_Cairo_Tile> wrap_target(
-			new AsyncTarget_Cairo_Tile(
-					etl::handle<synfig::Target_Cairo_Tile>::cast_dynamic(target_)
-			)
+			new AsyncTarget_Cairo_Tile(cast_target)
 		);
 		
 		signal_stop_.connect(sigc::mem_fun(*wrap_target,&AsyncTarget_Cairo_Tile::set_dead));
 		
 		target=wrap_target;
 	}
-	else if(etl::handle<synfig::Target_Cairo>::cast_dynamic(target_))
+	else if(auto cast_target = etl::handle<synfig::Target_Cairo>::cast_dynamic(target_))
 	{
 		etl::handle<AsyncTarget_Cairo> wrap_target(
-			new AsyncTarget_Cairo(
-				etl::handle<synfig::Target_Cairo>::cast_dynamic(target_)
-			)
+			new AsyncTarget_Cairo(cast_target)
 		);
 		
 		signal_stop_.connect(sigc::mem_fun(*wrap_target,&AsyncTarget_Cairo::set_dead));
@@ -825,10 +790,7 @@ AsyncRenderer::stop()
 			signal_stop_();
 
 #if REJOIN_ON_STOP
-#ifdef SINGLE_THREADED
-			if (!single_threaded())
-#endif
-				render_thread->join();
+			render_thread->join();
 #endif
 			finish_time.assign_current_time();
 			finish_clock = ::clock();
@@ -837,15 +799,23 @@ AsyncRenderer::stop()
 			// Make sure all the dispatch crap is cleared out
 			//Glib::MainContext::get_default()->iteration(false);
 
-			if(success)
+			std::string error_message;
+			if(status == RENDERING_SUCCESS)
 				signal_success_();
+			else
+				error_message = _("Animation couldn't be rendered");
 
 			target=0;
 			render_thread=0;
-			
+
 			lock.release();
-			
-			signal_finished_();
+
+			if (status == RENDERING_ERROR) {
+				if (ProgressLogger *logger = dynamic_cast<ProgressLogger*>(cb))
+					error_message += "\n" + logger->get_error_message();
+			}
+
+			signal_finished_(error_message);
 		}
 	}
 }
@@ -863,6 +833,7 @@ AsyncRenderer::resume()
 void
 AsyncRenderer::start()
 {
+	App::dock_info_->set_render_progress(0.0);
 	start_time.assign_current_time();
 	finish_time = start_time;
 	start_clock = ::clock();
@@ -876,47 +847,24 @@ AsyncRenderer::start()
 	);
 }
 
-#ifdef SINGLE_THREADED
-void
-AsyncRenderer::rendering_progress()
-{
-	updating = true;
-	while(studio::App::events_pending()) studio::App::iteration(false);
-	updating = false;
-}
-#endif
-
 void
 AsyncRenderer::start_()
 {
-	error=false;success=false;
+	status = RENDERING_UNDEFINED;
 	if(target)
 	{
 #ifndef GLIB_DISPATCHER_BROKEN
 		done_connection=signal_done_.connect(mem_fun(*this,&AsyncRenderer::stop));
 #endif
-
-#ifdef SINGLE_THREADED
-		if (single_threaded())
-		{
-			//synfig::info("%s:%d rendering in the same thread", __FILE__, __LINE__);
-			target->signal_progress().connect(sigc::mem_fun(this,&AsyncRenderer::rendering_progress));
-			render_thread = (Glib::Thread*)1;
-			render_target();
-		}
-		else
-#endif
-		{
-			render_thread=Glib::Thread::create(
-				sigc::mem_fun(*this,&AsyncRenderer::render_target),
+		render_thread=Glib::Thread::create(
+			sigc::mem_fun(*this,&AsyncRenderer::render_target),
 #if REJOIN_ON_STOP
-				true
+			true
 #else
-				false
+			false
 #endif
-				);
-			assert(render_thread);
-		}
+			);
+		assert(render_thread);
 	}
 	else
 	{
@@ -929,13 +877,24 @@ AsyncRenderer::render_target()
 {
 	etl::handle<Target> target(AsyncRenderer::target);
 
-	if(target && target->render())
-	{
-		success=true;
-	}
-	else
-	{
-		error=true;
+	try {
+		if(target && target->render(cb))
+		{
+			status = RENDERING_SUCCESS;
+		}
+		else
+		{
+			status = RENDERING_ERROR;
+#ifndef REJOIN_ON_STOP
+			return;
+#endif
+		}
+	} catch (...) {
+		status = RENDERING_ERROR;
+		string error_str = _("Internal error: some exception has been thrown while rendering");
+		synfig::error(error_str);
+		if (cb)
+			cb->error(error_str);
 #ifndef REJOIN_ON_STOP
 		return;
 #endif
