@@ -47,6 +47,8 @@
 
 #include <gui/localization.h>
 
+#include "gui/resourcehelper.h"
+
 #endif
 
 /* === U S I N G =========================================================== */
@@ -61,22 +63,13 @@ using namespace studio;
 #define VERSION	"unknown"
 #endif
 
-#ifdef _WIN32
-// Do we really need this nested ifdef??
-#	ifdef IMAGE_DIR
-#		define IMAGE_DIR "share\\pixmaps"
-#	endif
-#endif
-
-#ifndef IMAGE_DIR
-#	define IMAGE_DIR "/usr/local/share/pixmaps"
-#endif
-
 #ifndef IMAGE_EXT
 #	define IMAGE_EXT	"png"
 #endif
 
-#define stringify(x) #x
+//#define stringify(x) #x
+#define stringify(x) (x)
+//#define stringify(x) (std::to_string(x).c_str())
 
 /* === G L O B A L S ======================================================= */
 
@@ -91,10 +84,10 @@ About::About()
 	set_version(VERSION);
 	set_comments(_("2D vector animation studio"));
 
-	set_website("http://synfig.org/");
+	set_website("https://synfig.org/");
 	set_website_label(_("Visit the Synfig website"));
 
-	set_copyright(_("Copyright (c) 2001-2017\nSynfig developers & contributors"));
+	set_copyright(_("Copyright (c) 2001-2019\nSynfig developers & contributors"));
 	Glib::ustring license =
 		"This program is free software; you can redistribute it and/or modify "
 		"it under the terms of the GNU General Public License as published by "
@@ -173,29 +166,71 @@ About::About()
 	artists.push_back("Robert B. Quattlebaum Jr. (darco)");
 	artists.push_back("Thimotee Guiet (satrip)");
 	artists.push_back("Yu Chen (jcome)");
+	artists.push_back("");
 	set_artists(artists);
 
+	Glib::ustring section_name = "Vectorizer feature";
+	std::vector<Glib::ustring> credits;
+	credits.push_back("Credits:");
+	Glib::ustring credit_header =
+		"Vectorizer feature is ported from OpenToonz open-source animation\n"
+		"software, which is developed from Toonz, a software originally\n"
+		"created by Digital Video, S.p.A., Rome Italy.";
+
+	credits.push_back(credit_header);
+	credits.push_back("https://github.com/opentoonz/opentoonz/\n");
+	
+	credits.push_back("Copyright (c) 2016 - 2019, DWANGO Co., Ltd.");
+	credits.push_back("Copyright (c) 2016 Toshihiro Shimizu https://github.com/meso");
+	credits.push_back("Copyright (c) 2016 Shinya Kitaoka https://github.com/skitaoka");
+	credits.push_back("Copyright (c) 2016 shun-iwasawa https://github.com/shun-iwasawa");
+	credits.push_back("Copyright (c) 2016 Campbell Barton https://github.com/ideasman42");
+	credits.push_back("Copyright (c) 2019 luzpaz https://github.com/luzpaz");
+	credits.push_back("Copyright (c) 2019 - 2020, Ankit Kumar Dwivedi https://github.com/ankit-kumar-dwivedi");
+	
+	Glib::ustring credit_footer =
+		"\n== Vectorizer License ==\n\n"
+		"Vectorizer feature code is licensed under BSD 3-Clause \"New\" or \"Revised\"   \n"
+		"License. Redistribution and use in source and binary forms, with or             \n"
+		"without modification, are permitted provided that the following conditions      \n"
+		"are met:\n"
+		"\t1. Redistributions of source code must retain the above copyright notice,     \n"
+		"\t    this list of conditions and the following disclaimer.\n"
+		"\t2. Redistributions in binary form must reproduce the above copyright\n"
+		"\t    notice, this list of conditions and the following disclaimer in the\n"
+		"\t    documentation and/or other materials provided with the distribution.      \n"
+		"\t3. Neither the name of the copyright holder nor the names of its\n"
+		"\t    contributors may be used to endorse or promote products derived \n"
+		"\t    from this software without specific prior written permission.\n"
+		"\n"
+		"THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND\n"
+		"CONTRIBUTORS \"AS IS\" AND ANY EXPRESS OR IMPLIED\n"
+		"WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED\n"
+		"WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR\n"
+		"PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT\n"
+		"HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,\n"
+		"INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES\n"
+		"(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE\n"
+		"GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS\n"
+		"INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,\n"
+		"WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING\n"
+		"NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE\n"
+		"OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF \n"
+		"SUCH DAMAGE.\n\n"
+		"== Vectorizer License End ==\n";
+
+	credits.push_back(credit_footer);
+	add_credit_section(section_name,credits);
+	
 	// TRANSLATORS: change this to your name, separate multiple names with \n
 	set_translator_credits(_("translator-credits"));
 
-	std::string imagepath;
-#ifdef _WIN32
-	imagepath=App::get_base_path()+ETL_DIRECTORY_SEPARATOR+IMAGE_DIR;
-#else
-	imagepath=IMAGE_DIR;
-#endif
-	char* synfig_root=getenv("SYNFIG_ROOT");
-	if(synfig_root) {
-		imagepath=synfig_root;
-		imagepath+=ETL_DIRECTORY_SEPARATOR;
-		imagepath+="share";
-		imagepath+=ETL_DIRECTORY_SEPARATOR;
-		imagepath+="pixmaps";
-	}
-	imagepath+=ETL_DIRECTORY_SEPARATOR;
+	std::string imagepath = ResourceHelper::get_image_path("synfig_icon." IMAGE_EXT);
 
 	Gtk::Image *Logo = manage(new class Gtk::Image());
-	Logo->set(imagepath+"synfig_icon." IMAGE_EXT);
+	
+	Logo->set(imagepath);
+	Logo->set_parent(*this);
 	set_logo(Logo->get_pixbuf());
 
 #ifdef SHOW_EXTRA_INFO
