@@ -30,6 +30,8 @@
 #	include <config.h>
 #endif
 
+#include <synfig/general.h>
+
 #include "docks/dockmanager.h"
 #include <stdexcept>
 #include "docks/dockable.h"
@@ -39,7 +41,7 @@
 #include <synfigapp/main.h>
 #include <gdkmm/general.h>
 
-#include "general.h"
+#include <gui/localization.h>
 
 #include <gtkmm/paned.h>
 #include <gtkmm/box.h>
@@ -123,12 +125,12 @@ class studio::DockSettings : public synfigapp::Settings
 public:
 	DockSettings(DockManager* dock_manager):dock_manager(dock_manager)
 	{
-		synfigapp::Main::settings().add_domain(this,"dock");
+		synfigapp::Main::settings().add_domain(this,"workspace");
 	}
 
 	virtual ~DockSettings()
 	{
-		synfigapp::Main::settings().remove_domain("dock");
+		synfigapp::Main::settings().remove_domain("workspace");
 	}
 
 	virtual bool get_value(const synfig::String& key_, synfig::String& value)const
@@ -191,22 +193,19 @@ void
 DockManager::register_dockable(Dockable& x)
 {
 	dockable_list_.push_back(&x);
-	// synfig::info("DockManager::register_dockable(): Registered dockable \"%s\"",dockable_list_.back()->get_name().c_str());
 	signal_dockable_registered()(&x);
 }
 
 bool
 DockManager::unregister_dockable(Dockable& x)
 {
-	std::list<Dockable*>::iterator iter;
-	for(iter=dockable_list_.begin();iter!=dockable_list_.end();++iter)
+	for(std::list<Dockable*>::iterator iter = dockable_list_.begin(); iter != dockable_list_.end(); ++iter)
 	{
-		if(&x==*iter)
+		if (&x == *iter)
 		{
 			remove_widget_recursive(x);
 			dockable_list_.erase(iter);
 			signal_dockable_unregistered()(&x);
-			synfig::info("DockManager::unregister_dockable(): \"%s\" has been Unregistered",x.get_name().c_str());
 			update_window_titles();
 			return true;
 		}
@@ -319,7 +318,7 @@ DockManager::remove_empty_container_recursive(Gtk::Container &container)
 	if (window)
 	{
 		if (!window->get_child())
-			window->hide();
+			window->close();
 	}
 	else
 	if (book)
@@ -362,7 +361,7 @@ DockManager::add_widget(Gtk::Widget &dest_widget, Gtk::Widget &src_widget, bool
 	remove_widget_recursive(src_widget);
 
 	// create new paned and link all
-	Gtk::Paned *paned = manage(vertical ? (Gtk::Paned*)new Gtk::VPaned() : (Gtk::Paned*)new Gtk::HPaned());
+	Gtk::Paned *paned = manage(new Gtk::Paned(vertical ? Gtk::ORIENTATION_VERTICAL : Gtk::ORIENTATION_HORIZONTAL));
 	paned->show();
 	DockLinkPoint(paned, first).link(src_widget);
 	DockLinkPoint(paned, !first).link(dest_widget);
@@ -401,6 +400,17 @@ std::string DockManager::read_string(std::string &x)
 	return res;
 }
 
+std::string DockManager::extract_dockable_name(std::string &x) const
+{
+	size_t pos = x.find(",");
+	std::string res = x.substr(0, pos);
+	if (pos == std::string::npos)
+		x.clear();
+	else
+		x = x.substr(pos+1); // skip comma
+	return res;
+}
+
 int DockManager::read_int(std::string &x)
 {
 	return strtol(read_string(x).c_str(), NULL, 10);
@@ -439,7 +449,7 @@ Gtk::Widget* DockManager::read_widget(std::string &x)
 		if (!first && second) return second;
 
 		// create paned
-		Gtk::Paned *paned = manage(hor ? (Gtk::Paned*)new Gtk::HPaned() : (Gtk::Paned*)new Gtk::VPaned());
+		Gtk::Paned *paned = manage(new Gtk::Paned(vert ? Gtk::ORIENTATION_VERTICAL : Gtk::ORIENTATION_HORIZONTAL));
 		paned->pack1(*first,  true, false);
 		paned->pack2(*second, true, false);
 		paned->set_position(size);
@@ -456,22 +466,21 @@ Gtk::Widget* DockManager::read_widget(std::string &x)
 		DockBook *book = NULL;
 		do
 		{
-			std::string name = read_string(x);
+			std::string dockable_name_params = read_string(x);
+			std::string name = extract_dockable_name(dockable_name_params);
 			if (!name.empty())
 			{
-				Dockable *dockable = &find_dockable(name);
-				if (dockable != NULL)
-				{
-					Gtk::Container *container = dockable->get_parent();
-					if (container)
-					{
-						container->remove(*dockable);
-						containers_to_remove_[container] = true;
-					}
-					if (book == NULL) { book = manage(new DockBook()); book->show(); }
-					book->add(*dockable);
+				Dockable &dockable = find_dockable(name);
+				if (!dockable_name_params.empty())
+					dockable.read_layout_string(dockable_name_params);
+				Gtk::Container *container = dockable.get_parent();
+				if (container) {
+					container->remove(dockable);
+					containers_to_remove_[container] = true;
 				}
-			}
+				if (book == NULL) { book = manage(new DockBook()); book->show(); }
+				book->add(dockable);
+			}			
 		} while (read_separator(x));
 
 		return book;
@@ -564,7 +573,6 @@ void DockManager::write_bool(std::string &x, bool b)
 void DockManager::write_widget(std::string &x, Gtk::Widget* widget)
 {
 	Gtk::Paned *paned = dynamic_cast<Gtk::Paned*>(widget);
-	Gtk::HPaned *hpaned = dynamic_cast<Gtk::HPaned*>(widget);
 	DockBook *book = dynamic_cast<DockBook*>(widget);
 	DockDialog *dialog = dynamic_cast<DockDialog*>(widget);
 
@@ -618,7 +626,7 @@ void DockManager::write_widget(std::string &x, Gtk::Widget* widget)
 	else
 	if (paned)
 	{
-		write_string(x, hpaned ? "[hor|" : "[vert|");
+		write_string(x, paned->get_orientation() == Gtk::ORIENTATION_HORIZONTAL ? "[hor|" : "[vert|");
 		write_int(x, paned->get_position());
 		write_separator(x);
 		write_widget(x, paned->get_child1());
@@ -636,7 +644,17 @@ void DockManager::write_widget(std::string &x, Gtk::Widget* widget)
 			if (dockable)
 			{
 				write_separator(x);
-				write_string(x, dockable->get_name());
+				std::string name_params = dockable->get_name();
+				std::string params;
+				dockable->write_layout_string(params);
+				if (!params.empty()) {
+					if (params.find_first_of("]|") != std::string::npos) {
+						synfig::warning("Ignoring %s's layout info: it must not have ] or | characters.", dockable->get_name().c_str());
+					} else {
+						name_params += "," + params;
+					}
+				}
+				write_string(x, name_params);
 			}
 		}
 		write_separator(x, false);
@@ -705,6 +723,17 @@ std::string DockManager::layout_from_template(const std::string &tpl, float dx,
 	return res;
 }
 
+void DockManager::set_dock_area_visibility(bool visible, DockBook* source)
+{
+	for(auto iter=dockable_list_.begin();iter!=dockable_list_.end();++iter) {
+		Dockable * dockable = *iter;
+		if (!dockable->is_visible())
+			continue;
+		DockBook * book = dynamic_cast<DockBook*>((*iter)->get_parent());
+		if (book)
+			book->set_dock_area_visibility(visible, source);
+	}
+}
 
 void
 DockManager::update_window_titles()
