@@ -43,6 +43,7 @@
 #include <sigc++/sigc++.h>
 
 #include <giomm.h>
+#include <fstream>
 
 #include <gtkmm/stock.h>
 #include <gtkmm/image.h>
@@ -56,11 +57,12 @@
 
 #include <synfig/savecanvas.h>
 #include <synfig/canvasfilenaming.h>
-#include <synfig/layers/layer_pastecanvas.h>
+#include <synfig/layers/layer_switch.h>
 #include <synfig/valuenode_registry.h>
 #include <synfig/valuenodes/valuenode_composite.h>
 #include <synfig/valuenodes/valuenode_duplicate.h>
 #include <synfig/widthpoint.h>
+#include <synfig/zstreambuf.h>
 
 #include "instance.h"
 #include "canvasview.h"
@@ -93,7 +95,7 @@ int studio::Instance::instance_count_=0;
 
 /* === M E T H O D S ======================================================= */
 
-Instance::Instance(synfig::Canvas::Handle canvas, synfig::FileSystemTemporary::Handle container):
+Instance::Instance(synfig::Canvas::Handle canvas, synfig::FileSystem::Handle container):
 	synfigapp::Instance		(canvas, container),
 	canvas_tree_store_		(Gtk::TreeStore::create(canvas_tree_model)),
 	history_tree_store_		(HistoryTreeStore::create(this)),
@@ -108,14 +110,6 @@ Instance::Instance(synfig::Canvas::Handle canvas, synfig::FileSystemTemporary::H
 	signal_undo_status().connect(sigc::mem_fun(*this,&studio::Instance::set_undo_status));
 	signal_redo_status().connect(sigc::mem_fun(*this,&studio::Instance::set_redo_status));
 
-	signal_saved().connect(
-		sigc::hide_return(
-			sigc::ptr_fun(
-				studio::AutoRecover::auto_backup
-			)
-		)
-	);
-
 	refresh_canvas_tree();
 }
 
@@ -123,6 +117,35 @@ Instance::~Instance()
 {
 }
 
+// this function returns true if the given extension belongs to image layer type
+bool Instance::is_img(synfig::String ext) const
+{
+		std::set <String> img_ext{".jpg",".jpeg",".png",".bmp",".gif"};
+		bool is_in = img_ext.find(ext) != img_ext.end();
+		return is_in;
+}
+
+synfig::Layer::Handle
+Instance::layer_inside_switch(synfig::Layer_Switch::Handle paste) const
+{
+	synfig::Layer::Handle child_layer;
+	synfig::Canvas::Handle canvas = paste->get_sub_canvas();
+	synfig::String active_layer = "";
+	active_layer = paste->get_param("layer_name").get(synfig::String());
+	if(canvas)
+	{
+		for(IndependentContext i = canvas->get_independent_context(); *i; i++)
+		{
+			if((*i)->get_description()==active_layer)
+			{
+				child_layer = (*i);
+			}
+		}
+	}
+	return child_layer;
+}
+
+
 int
 Instance::get_visible_canvases()const
 {
@@ -135,7 +158,7 @@ Instance::get_visible_canvases()const
 }
 
 handle<Instance>
-Instance::create(synfig::Canvas::Handle canvas, synfig::FileSystemTemporary::Handle container)
+Instance::create(synfig::Canvas::Handle canvas, synfig::FileSystem::Handle container)
 {
 	// Construct a new instance
 	handle<Instance> instance(new Instance(canvas, container));
@@ -188,7 +211,7 @@ void
 Instance::set_undo_status(bool x)
 {
 	undo_status_=x;
-	App::dock_toolbox->update_tools();
+	App::dock_toolbox->refresh();
 	signal_undo_redo_status_changed()();
 }
 
@@ -196,7 +219,7 @@ void
 Instance::set_redo_status(bool x)
 {
 	redo_status_=x;
-	App::dock_toolbox->update_tools();
+	App::dock_toolbox->refresh();
 	signal_undo_redo_status_changed()();
 }
 
@@ -205,10 +228,10 @@ studio::Instance::run_plugin(std::string plugin_path)
 {
 	handle<synfigapp::UIInterface> uim = this->find_canvas_view(this->get_canvas())->get_ui_interface();
 
-	string message = strprintf(_("Do you realy want to run plugin for file \"%s\"?" ),
+	String message = strprintf(_("Do you really want to run plugin for file \"%s\"?" ),
 				this->get_canvas()->get_name().c_str());
 
-	string details = strprintf(_("This operation cannot be undone and all undo history will be cleared."));
+	String details = strprintf(_("This operation cannot be undone and all undo history will be cleared."));
 
 	int answer = uim->confirmation(
 				message,
@@ -217,68 +240,189 @@ studio::Instance::run_plugin(std::string plugin_path)
 				_("Proceed"),
 				synfigapp::UIInterface::RESPONSE_OK);
 
-	if(answer == synfigapp::UIInterface::RESPONSE_OK){
+	if(answer != synfigapp::UIInterface::RESPONSE_OK)
+		return;
 
-		OneMoment one_moment;
+	OneMoment one_moment;
 
-		Canvas::Handle canvas(this->get_canvas());
+	Canvas::Handle canvas(this->get_canvas());
 
-		synfigapp::PluginLauncher launcher(canvas);
+	backup(true);
+	FileSystemTemporary::Handle temporary_filesystem = FileSystemTemporary::Handle::cast_dynamic(get_canvas()->get_file_system());
+	String tmp_filename = temporary_filesystem->get_temporary_directory() + ETL_DIRECTORY_SEPARATOR + temporary_filesystem->get_temporary_filename_base();
 
-		Time cur_time;
-		cur_time = canvas->get_time();
+	Time cur_time;
+	cur_time = canvas->get_time();
 
-		this->close();
+	// Generate temporary file name
+	String filename_original = get_canvas()->get_file_name();
+	String filename_processed;
+	struct stat buf;
+	do {
+		synfig::GUID guid;
+		filename_processed = filename_original+"."+guid.get_string().substr(0,8)+".sif";
+	} while (stat(filename_processed.c_str(), &buf) != -1);
 
-		if(canvas->count()!=1)
+	close(false);
+
+	if(canvas->count()!=1)
+	{
+		one_moment.hide();
+		App::dialog_message_1b(
+				"ERROR",
+				_("The plugin operation has failed."),
+				_("This can be due to current file "
+					"being referenced by another composition that is already open, "
+					"or because of an internal error in Synfig Studio. Try closing "
+					"any compositions that might reference this file and try again, "
+					"or restart Synfig Studio."),
+				_("Close"));
+
+	} else {
+
+		// Save file copy
+		String filename_ext = filename_extension(filename_original);
+		if (filename_ext.empty())
+			filename_ext = ".sifz";
+		FileSystem::ReadStream::Handle stream_in = temporary_filesystem->get_read_stream("#project"+filename_ext);
+		if (!stream_in)
 		{
+			synfig::error(strprintf("run_plugin(): Unable to open file for reading - %s", temporary_filesystem->get_real_uri("#project"+filename_ext).c_str()));
+		}
+		if (filename_ext == ".sifz")
+			stream_in = new ZReadStream(stream_in);
+		std::ofstream  outfile(filename_processed, std::ios::binary);
+		outfile << stream_in->rdbuf();
+		outfile.close();
+		stream_in.reset();
+
+
+		bool result = false;
+		String output;
+		String command = "";
+
+		// Path to python binary can be overridden
+		// with SYNFIG_PYTHON_BINARY env variable:
+		const char* custom_python_binary=getenv("SYNFIG_PYTHON_BINARY");
+		if(custom_python_binary) {
+			command=custom_python_binary;
+			if (!App::check_python_version(command)) {
+				output=_("Error: You need to have Python 3 installed.");
+				command="";
+			}
+		} else {
+		// Set path to python binary depending on the os type.
+		// For Windows case Python binary is expected
+		// at INSTALL_PREFIX/python/python.exe
+			std::list< String > binary_choices;
+			binary_choices.push_back("python");
+			binary_choices.push_back("python3");
+			std::list< String >::iterator iter;
+			for(iter=binary_choices.begin();iter!=binary_choices.end();iter++)
+			{
+				String python_path;
+#ifdef _WIN32
+				python_path = "\"" + App::get_base_path()+ETL_DIRECTORY_SEPARATOR+"python"+ETL_DIRECTORY_SEPARATOR+*iter+".exe" + "\"";
+#else
+				python_path = *iter;
+#endif
+				if (App::check_python_version(python_path))
+				{
+					command = python_path;
+					break;
+				}
+
+			}
+
+		}
+		if (command.empty())
+		{
+			output=_("Error: No Python 3 binary found.\n\nHint: You can set SYNFIG_PYTHON_BINARY environment variable pointing at your custom python installation.");
+		} else {
+			synfig::info("Python 3 binary found: "+command);
+
+
+			// Construct the full command:
+			command = command+" \""+plugin_path+"\" \""+filename_processed+"\" 2>&1";
+#ifdef _WIN32
+			// This covers the dumb cmd.exe behavior.
+			// See: http://eli.thegreenplace.net/2011/01/28/on-spaces-in-the-paths-of-programs-and-files-on-windows/
+			command = "\"" + command + "\"";
+#endif
+
+			FILE* pipe = popen(command.c_str(), "r");
+			if (!pipe) {
+				output = "ERROR: pipe failed!";
+			} else {
+				char buffer[128];
+				while(!feof(pipe)) {
+					if(fgets(buffer, 128, pipe) != NULL)
+							output += buffer;
+				}
+
+				if (output != "" ){
+					synfig::info(output);
+				}
+
+				int exitcode=pclose(pipe);
+
+				if (0==exitcode){
+					result=true;
+				}
+			}
+		}
+
+		if (!result){
 			one_moment.hide();
 			App::dialog_message_1b(
-					"ERROR",
-					_("The plugin operation has failed."),
-					_("This can be due to current file "
-						"being referenced by another composition that is already open, "
-						"or because of an internal error in Synfig Studio. Try closing "
-						"any compositions that might reference this file and try again, "
-						"or restart Synfig Studio."),
+					"Error",
+					output,
+					"details",
 					_("Close"));
 
 			one_moment.show();
 
 		} else {
-			bool result;
-			result = launcher.execute( plugin_path, App::get_base_path() );
-			if (!result){
-				one_moment.hide();
-				App::dialog_message_1b(
-						"Error",
-						launcher.get_output(),
-						"details",
-						_("Close"));
-
-				one_moment.show();
+			// Restore file copy
+			FileSystem::WriteStream::Handle stream = temporary_filesystem->get_write_stream("#project"+filename_ext);
+			if (stream && filename_ext == ".sifz")
+				stream = FileSystem::WriteStream::Handle(new ZWriteStream(stream));
 
+			if (!stream)
+			{
+				synfig::error("run_plugin(): Unable to open file for write");
+			} else {
+				std::ifstream  infile(filename_processed, std::ios::binary);
+				*stream << infile.rdbuf();
+				infile.close();
+				stream.reset();
+				remove(filename_processed.c_str());
 			}
 		}
-
-
-		canvas=0;
-
-		App::open_as(launcher.get_result_path(),launcher.get_original_path());
-
-
-		etl::handle<Instance> new_instance = App::instance_list.back();
-		new_instance->inc_action_count(); // This file isn't saved! mark it as such
-
-		// Restore time cursor position
-		canvas = App::instance_list.back()->get_canvas();
-		etl::handle<synfigapp::CanvasInterface> new_canvas_interface(new_instance->find_canvas_interface(canvas));
-		new_canvas_interface->set_time(cur_time);
-
 	}
-	return;
-}
 
+	canvas=0;
+
+	bool ok = App::open_from_temporary_filesystem(tmp_filename);
+	if (!ok) {
+		synfig::error("run_plugin(): Cannot reopen file");
+		return;
+	}
+
+	etl::handle<Instance> new_instance = App::instance_list.back();
+	if (!new_instance) {
+		synfig::error("run_plugin(): Cannot retrieve new instance");
+	} else {
+		// Restore time cursor position
+		canvas = new_instance->get_canvas();
+		etl::handle<synfigapp::CanvasInterface> new_canvas_interface(new_instance->find_canvas_interface(canvas));
+		if (!new_canvas_interface) {
+			synfig::error("run_plugin(): Cannot retrieve canvas interface");
+		} else {
+			new_canvas_interface->set_time(cur_time);
+		}
+	}
+}
 
 bool
 studio::Instance::save_as(const synfig::String &file_name)
@@ -286,10 +430,32 @@ studio::Instance::save_as(const synfig::String &file_name)
 	if(synfigapp::Instance::save_as(file_name))
 	{
 		// after changing the filename, update the render settings with the new filename
-		list<handle<CanvasView> >::iterator iter;
-		for(iter=canvas_view_list().begin();iter!=canvas_view_list().end();iter++)
+		for(list<handle<CanvasView> >::iterator iter = canvas_view_list().begin(); iter!=canvas_view_list().end(); iter++)
 			(*iter)->render_settings.set_entry_filename();
 		App::add_recent_file(etl::handle<Instance>(this));
+
+		// check for unsaved layers (external bitmaps, etc)
+		std::vector<Layer::Handle> layers;
+		find_unsaved_layers(layers);
+		if (!layers.empty())
+		{
+			String message = _("Current canvas saved, but unable to save data for following layers:");
+			String details;
+			for(std::vector<Layer::Handle>::const_iterator i = layers.begin(); i != layers.end(); ++i)
+			{
+				details.append((*i)->get_description());
+				if ( Layer_Bitmap::Handle::cast_dynamic(*i)
+			  	  && (*i)->get_param_list().count("filename") )
+			  	{
+			  		ValueBase value = (*i)->get_param("filename");
+			  		if (value.same_type_as(String()))
+			  			details.append(" (" + value.get(String()) + ")");
+			  	}
+				details.append("\n");
+			}
+			App::dialog_message_1b("WARNING", message, "details", _("Close"), details);
+		}
+
 		return true;
 	}
 	return false;
@@ -313,19 +479,18 @@ studio::Instance::save()
 			return STATUS_CANCEL;
 	}
 
-	if (synfigapp::Instance::save())
+	if (!save_as(get_canvas()->get_file_name()))
 	{
-		App::add_recent_file(etl::handle<Instance>(this));
-		return STATUS_OK;
+		string msg(strprintf(_("Unable to save to '%s'"), get_file_name().c_str()));
+		App::dialog_message_1b(
+				"ERROR",
+				msg.c_str(),
+				"details",
+				_("Close"));
+		return STATUS_ERROR;
 	}
-	string msg(strprintf(_("Unable to save to '%s'"), get_file_name().c_str()));
-	App::dialog_message_1b(
-			"ERROR",
-			msg.c_str(),
-			"details",
-			_("Close"));
 
-	return STATUS_ERROR;
+	return STATUS_OK;
 }
 
 // the filename will be set to "Synfig Animation 1" or some such when first created
@@ -476,7 +641,7 @@ Instance::update_all_titles()
 }
 
 void
-Instance::close()
+Instance::close(bool remove_temporary_files)
 {
 	// This will increase the reference count so we don't get DELETED
 	// until we are ready
@@ -502,9 +667,6 @@ Instance::close()
 	if(studio::App::get_selected_instance()==this)
 		studio::App::set_selected_instance(0);
 
-	// Turn-off/clean-up auto recovery
-	studio::App::auto_recover->clear_backup(get_canvas());
-
 	// Remove us from the active instance list
 	std::list<etl::handle<studio::Instance> >::iterator iter;
 	for(iter=studio::App::instance_list.begin();iter!=studio::App::instance_list.end();iter++)
@@ -522,7 +684,7 @@ Instance::close()
 		(*iter)->hide();
 
 	// Consume pending events before deleting the canvas views
-	while(studio::App::events_pending())studio::App::iteration(true);
+	App::process_all_events();
 
 	// Delete all of the canvas views
 	canvas_view_list().clear();
@@ -538,6 +700,11 @@ Instance::close()
 	{
 		studio::App::instance_list.front()->canvas_view_list().front()->present();
 	}
+
+	if (remove_temporary_files) {
+		FileSystemTemporary::Handle temporary_filesystem = FileSystemTemporary::Handle::cast_dynamic(get_canvas()->get_file_system());
+		temporary_filesystem->discard_changes();
+	}
 }
 
 void
@@ -1119,10 +1286,12 @@ Instance::process_action(synfig::String name, synfigapp::Action::ParamList param
 				// export and rename value dialog
 				if (entry.name == "ValueNodeRename") button2 = _("Rename");
 				// set layer description dialog
-				if (entry.name == "LayerSetDesc")
+				else if (entry.name == "LayerSetDesc")
 				{
-					button2 = _("Set");
+					button2 = _("OK");
 					label = _("Description: ");
+				} else if (entry.name == "GroupAddLayers") {
+					button2 = _("Add");
 				}
 
 				if(!studio::App::dialog_entry(entry.local_name,
@@ -1373,6 +1542,8 @@ Instance::make_param_menu(Gtk::Menu *menu,synfig::Canvas::Handle canvas, synfiga
 		ADD_IMAGE_MENU_ITEM(TYPE_SQUARED, "synfig-squared_interpolation", "Cusp Before: Squared")
 		ADD_IMAGE_MENU_ITEM(TYPE_PEAK, "synfig-peak_interpolation", "Cusp Before: Peak")
 		ADD_IMAGE_MENU_ITEM(TYPE_FLAT, "synfig-flat_interpolation", "Cusp Before: Flat")
+		ADD_IMAGE_MENU_ITEM(TYPE_INNER_ROUNDED, "synfig-rounded_interpolation", "Cusp Before: Inner Rounded")
+		ADD_IMAGE_MENU_ITEM(TYPE_INNER_PEAK, "synfig-peak_interpolation", "Cusp Before: Off-Peak")
 
 		///////
 		item = Gtk::manage(new Gtk::SeparatorMenuItem());
@@ -1388,6 +1559,8 @@ Instance::make_param_menu(Gtk::Menu *menu,synfig::Canvas::Handle canvas, synfiga
 		ADD_IMAGE_MENU_ITEM(TYPE_SQUARED, "synfig-squared_interpolation", "Cusp After: Squared")
 		ADD_IMAGE_MENU_ITEM(TYPE_PEAK, "synfig-peak_interpolation", "Cusp After: Peak")
 		ADD_IMAGE_MENU_ITEM(TYPE_FLAT, "synfig-flat_interpolation", "Cusp After: Flat")
+		ADD_IMAGE_MENU_ITEM(TYPE_INNER_ROUNDED, "synfig-rounded_interpolation", "Cusp After: Inner Rounded")
+		ADD_IMAGE_MENU_ITEM(TYPE_INNER_PEAK, "synfig-peak_interpolation", "Cusp After: Off-Peak")
 
 		///////
 		item = Gtk::manage(new Gtk::SeparatorMenuItem());
@@ -1581,14 +1754,13 @@ Instance::make_param_menu(Gtk::Menu *menu,synfig::Canvas::Handle canvas,const st
 		make_param_menu(menu,canvas,value_desc, 0.f, false);
 }
 
-
 void
 Instance::gather_uri(std::set<synfig::String> &x, const synfig::ValueNode::Handle &value_node) const
-{
+{	//check for null value
 	if (!value_node || !value_node->get_parent_canvas()) return;
-
+	
 	LinkableValueNode::Handle linkable_value_node = LinkableValueNode::Handle::cast_dynamic(value_node);
-	if (!linkable_value_node) return;
+	if (!linkable_value_node) return;	//check that linkable_value_node is not empty
 
 	FileSystem::Handle file_system = value_node->get_parent_canvas()->get_file_system();
 	if (!file_system) return;
@@ -1620,7 +1792,7 @@ void
 Instance::gather_uri(std::set<synfig::String> &x, const synfig::Layer::Handle &layer) const
 {
 	if (!layer || !layer->get_canvas()) return;
-
+	
 	FileSystem::Handle file_system = layer->get_canvas()->get_file_system();
 	if (!file_system) return;
 
@@ -1632,7 +1804,9 @@ Instance::gather_uri(std::set<synfig::String> &x, const synfig::Layer::Handle &l
 			ValueBase v = layer->get_param(i->get_name());
 			if (v.can_get(String()))
 			{
-				String filename = CanvasFileNaming::make_full_filename(layer->get_canvas()->get_file_name(), v.get(String()));
+				String filename = v.get(String());
+				if (!filename.empty() && filename[0] != '#')
+					filename = etl::absolute_path(layer->get_canvas()->get_file_path() + "/", filename);
 				String uri = file_system->get_real_uri(filename);
 				if (!uri.empty()) x.insert(uri);
 			}
@@ -1642,6 +1816,10 @@ Instance::gather_uri(std::set<synfig::String> &x, const synfig::Layer::Handle &l
 		if (j != layer->dynamic_param_list().end() && j->second)
 			gather_uri(x, j->second);
 	}
+	
+	// This required to allow "Edit Image/Open File" commands to appear when clicked on a switch group
+	if (etl::handle<Layer_Switch> layer_switch = etl::handle<Layer_Switch>::cast_dynamic(layer))
+		gather_uri(x, layer_inside_switch(layer_switch));
 
 	//if (etl::handle<Layer_PasteCanvas> paste = etl::handle<Layer_PasteCanvas>::cast_dynamic(layer))
 	//	gather_uri(x, paste->get_param("canvas").get(Canvas::Handle()));
@@ -1659,6 +1837,9 @@ Instance::gather_uri(std::set<synfig::String> &x, const synfig::Canvas::Handle &
 		gather_uri(x, *i);
 }
 
+// this is the second function for gather_uri - call it gather_uri(set, layerlist)
+// iterate through all the layers and call gather_uri(set, layer) for each saperate layer 
+
 void
 Instance::gather_uri(std::set<synfig::String> &x, const synfigapp::SelectionManager::LayerList &layers) const
 {
@@ -1666,6 +1847,7 @@ Instance::gather_uri(std::set<synfig::String> &x, const synfigapp::SelectionMana
 		gather_uri(x, *i);
 }
 
+// this is the entry point for gather_uri - call it gather_uri(map, layerlist)
 void
 Instance::gather_uri(std::map<synfig::String, synfig::String> &x, const synfigapp::SelectionManager::LayerList &layers) const
 {
@@ -1708,15 +1890,54 @@ Instance::add_special_layer_actions_to_menu(Gtk::Menu *menu, const synfigapp::Se
 	gather_uri(uris, layers);
 	for(std::map<String, String>::const_iterator i = uris.begin(); i != uris.end(); ++i)
 	{
-		Gtk::MenuItem *item = manage(new Gtk::ImageMenuItem(Gtk::Stock::OPEN));
-		item->set_label( (String(_("Open file")) + " '" + i->first + "'").c_str() );
-		item->signal_activate().connect(
-			sigc::bind(sigc::ptr_fun(&App::open_uri), i->second) );
-		item->show();
-		menu->append(*item);
+		if(is_img(filename_extension(i->second)))// check if layer is image
+		{
+			Gtk::MenuItem *item = manage(new Gtk::ImageMenuItem(Gtk::Stock::OPEN));
+			item->set_label( (String(_("Edit image in external tool..."))).c_str() );
+			item->signal_activate().connect(
+				sigc::bind(sigc::ptr_fun(&App::open_img_in_external), i->second) );
+			item->show();
+			menu->append(*item);
+		}
+		else
+		{
+			Gtk::MenuItem *item = manage(new Gtk::ImageMenuItem(Gtk::Stock::OPEN));
+			item->set_label( (String(_("Open file")) + " '" + i->first + "'").c_str() );
+			item->signal_activate().connect(
+				sigc::bind(sigc::ptr_fun(&App::open_uri), i->second) );
+			item->show();
+			menu->append(*item);	
+		}
+	}
+	if(layers.size()==1)
+	{
+		if(etl::handle<Layer_Bitmap> my_layer_bitmap = etl::handle<Layer_Bitmap>::cast_dynamic(layers.front()))
+		{
+				Gtk::MenuItem *item2 = manage(new Gtk::ImageMenuItem(Gtk::Stock::CONVERT));
+				item2->set_label( (String(_("Convert to Vector"))).c_str() );
+				item2->signal_activate().connect(
+					sigc::bind(sigc::ptr_fun(&App::open_vectorizerpopup), my_layer_bitmap,layers.front()) );
+				item2->show();
+				menu->append(*item2);
+		}
+		else if(etl::handle<Layer_PasteCanvas> reference_layer = etl::handle<Layer_PasteCanvas>::cast_dynamic(layers.front()))
+		{
+			//the layer selected is a switch group
+			if(etl::handle<Layer_Bitmap> my_layer_bitmap = etl::handle<Layer_Bitmap>::cast_dynamic(layer_inside_switch(reference_layer)))
+			{
+				Gtk::MenuItem *item2 = manage(new Gtk::ImageMenuItem(Gtk::Stock::CONVERT));
+				item2->set_label( (String(_("Convert to Vector menu"))).c_str() );
+				item2->signal_activate().connect(
+					sigc::bind(sigc::ptr_fun(&App::open_vectorizerpopup), my_layer_bitmap,layers.front()) );
+				item2->show();
+				menu->append(*item2);
+			} 
+		}
 	}
 }
 
+// called whenever we right click any layer under layers panel
+// arguments - action_group: the current group of actions on the right click menu, layers: layerlist(because we can select multiple layer and then right click) 
 void
 Instance::add_special_layer_actions_to_group(const Glib::RefPtr<Gtk::ActionGroup>& action_group, synfig::String& ui_info, const synfigapp::SelectionManager::LayerList &layers) const
 {
@@ -1726,13 +1947,60 @@ Instance::add_special_layer_actions_to_group(const Glib::RefPtr<Gtk::ActionGroup
 	for(std::map<String, String>::const_iterator i = uris.begin(); i != uris.end(); ++i, ++index)
 	{
 		String action_name = etl::strprintf("special-action-open-file-%d", index);
-		String local_name = String(_("Open file")) + " '" + i->first + "'";
-		action_group->add(
-			Gtk::Action::create(
-				action_name,
-				Gtk::Stock::OPEN,
-				local_name, local_name ),
-			sigc::bind(sigc::ptr_fun(&App::open_uri), i->second) );
-		ui_info += strprintf("<menuitem action='%s' />", action_name.c_str());
+		//if the import layer is type image 
+		if(is_img(filename_extension(i->second)))
+		{
+			String local_name = String(_("Edit image in external tool..."));
+			action_group->add(
+				Gtk::Action::create(
+					action_name,
+					Gtk::Stock::OPEN,
+					local_name, local_name ),
+				sigc::bind(sigc::ptr_fun(&App::open_img_in_external), i->second) ); 
+			ui_info += strprintf("<menuitem action='%s' />", action_name.c_str());
+		}
+		else
+		{
+			String local_name = String(_("Open file")) + " '" + i->first + "'";
+			action_group->add(
+				Gtk::Action::create(
+					action_name,
+					Gtk::Stock::OPEN,
+					local_name, local_name ),
+				sigc::bind(sigc::ptr_fun(&App::open_uri), i->second) );
+			ui_info += strprintf("<menuitem action='%s' />", action_name.c_str());
+		}
+	}
+	if(layers.size()==1)
+	{
+		String local_name2 = String(_("Convert to Vector"));
+		String action_name2 = etl::strprintf("special-action-open-file-vectorizer-%d",index);
+		if(etl::handle<Layer_Switch> reference_layer = etl::handle<Layer_Switch>::cast_dynamic(layers.front()))
+		{
+			//the layer selected is a switch group
+			if(etl::handle<Layer_Bitmap> my_layer_bitmap = etl::handle<Layer_Bitmap>::cast_dynamic(layer_inside_switch(reference_layer)))
+			{
+				action_group->add(
+			 	Gtk::Action::create(
+			 		action_name2,
+			 		Gtk::Stock::CONVERT,
+			 		local_name2, local_name2 ),
+			 	sigc::bind(sigc::ptr_fun(&App::open_vectorizerpopup), my_layer_bitmap,layers.front()) );
+				 			ui_info += strprintf("<menuitem action='%s' />", action_name2.c_str());
+
+			} 
+		}
+		if(etl::handle<Layer_Bitmap> my_layer_bitmap = etl::handle<Layer_Bitmap>::cast_dynamic(layers.front()))
+		{
+				action_group->add(
+			 	Gtk::Action::create(
+			 		action_name2,
+			 		Gtk::Stock::CONVERT,
+			 		local_name2, local_name2 ),
+			 	sigc::bind(sigc::ptr_fun(&App::open_vectorizerpopup), my_layer_bitmap,layers.front()) );
+				 			ui_info += strprintf("<menuitem action='%s' />", action_name2.c_str());
+
+
+		}
 	}
 }
