@@ -54,15 +54,24 @@ namespace studio {
 		//! Constructor Helper - Initializes all of the menus
 		void init_menus();
 
+		//! Mandatory for constructing custom widgets from GTKBuilder files
+		void register_custom_widget_types();
+
 		static void show_dialog_input();
 		void on_recent_files_changed();
+		void on_custom_workspaces_changed();
 		void on_dockable_registered(Dockable* dockable);
 		void on_dockable_unregistered(Dockable* dockable);
 		void toggle_show_menubar();
 
-		Glib::RefPtr<Gtk::ToggleAction> toggle_menubar;
-		bool toggling_show_menubar;
+		guint save_workspace_merge_id;
+		guint custom_workspaces_merge_id;
 
+		void add_custom_workspace_menu_item_handlers();
+		void remove_custom_workspace_menu_item_handlers();
+
+	protected:
+		virtual bool on_key_press_event(GdkEventKey *key_event);
 
 	public:
 		MainWindow();
