@@ -30,21 +30,32 @@
 #	include <config.h>
 #endif
 
-#include "render.h"
-#include "app.h"
+#include <cerrno>
+#include <map>
+#include <glibmm.h>
+
 #include <gtkmm/frame.h>
 #include <gtkmm/alignment.h>
-#include <glibmm.h>
+
+#include <ETL/stringf>
+
+#include <synfig/general.h>
 #include <synfig/target_scanline.h>
 #include <synfig/canvas.h>
+#include <synfig/soundprocessor.h>
+
+#include "app.h"
 #include "asyncrenderer.h"
+#include "docks/dockmanager.h"
+#include "docks/dock_info.h"
 #include "dialogs/dialog_ffmpegparam.h"
 #include "dialogs/dialog_spritesheetparam.h"
 
-#include "general.h"
+#include "render.h"
 
-#include <ETL/stringf>
-#include <errno.h>
+#include <gui/localization.h>
+
+#include <gui/progresslogger.h>
 
 #endif
 
@@ -72,8 +83,9 @@ RenderSettings::RenderSettings(Gtk::Window& parent, etl::handle<synfigapp::Canva
 	entry_antialias(adjustment_antialias,1,0),
 	toggle_single_frame(_("Render _current frame only"), true),
 	toggle_extract_alpha(_("Extract alpha"), true),
-	tparam("libx264-lossless",2000)
+	tparam("mpeg4",6000)
 {
+	progress_logger.reset(new ProgressLogger());
 	tparam.sequence_separator=App::sequence_separator;
 	widget_rend_desc.show();
 	widget_rend_desc.signal_changed().connect(sigc::mem_fun(*this,&studio::RenderSettings::on_rend_desc_changed));
@@ -167,7 +179,7 @@ RenderSettings::RenderSettings(Gtk::Window& parent, etl::handle<synfigapp::Canva
 	toggle_single_frame.set_alignment(0, 0.5);
 	settings_table->attach(toggle_single_frame, 2, 3, 0, 1, Gtk::SHRINK|Gtk::FILL, Gtk::SHRINK|Gtk::FILL, 0, 0);
 	toggle_single_frame.set_active(false);
-	
+
 	toggle_extract_alpha.set_alignment(0, 0.5);
 	settings_table->attach(toggle_extract_alpha, 2, 3, 1, 2, Gtk::SHRINK|Gtk::FILL, Gtk::SHRINK|Gtk::FILL, 0, 0);
 	toggle_extract_alpha.set_active(false);
@@ -215,25 +227,41 @@ RenderSettings::set_entry_filename()
 			filename+=" ("+canvas->get_name()+')';
 	}
 
-	filename += ".png";
-
+	if (!is_absolute_path(filename))
+		filename = Glib::get_home_dir() + ETL_DIRECTORY_SEPARATOR + filename;
+	
 	try
 	{
-		entry_filename.set_text((filename));
+		if(!comboboxtext_target.get_active_row_number())
+			entry_filename.set_text(filename + ".avi");
+		// in case the file was saved and loaded again then .ext should be according to target
+		else on_comboboxtext_target_changed();
 	}
 	catch(...)
 	{
 		synfig::warning("Averted crash!");
-		entry_filename.set_text("output.png");
+		entry_filename.set_text("output.avi");
 	}
 }
 
 void
 RenderSettings::on_comboboxtext_target_changed()
 {
+	std::map<std::string,std::string> ext = {{"bmp",".bmp"}, {"cairo_png",".png"},{"dv",".dv"},
+					{"ffmpeg",".avi"},{"gif",".gif"},{"imagemagick",".png"}, {"jpeg",".jpg"},
+					{"magick++",".gif"},{"mng",".mng"},{"openexr",".exr"},{"png",".png"},
+					{"png-spritesheet",".png"},{"ppm",".ppm"}, {"yuv420p",".yuv"}, {"libav",".avi"}};
 	int i = comboboxtext_target.get_active_row_number();
 	if (i < 0 || i >= (int)target_names.size()) return;
 	if (target_name == target_names[i]) return;
+	auto itr = ext.find(target_names[i]); 
+    // check if target_name is there in map
+    if(itr != ext.end())
+	{
+		String filename = entry_filename.get_text();
+		String newfilename = filename.substr(0,filename.find_last_of('.'))+itr->second;
+		entry_filename.set_text(newfilename);
+	}
 	set_target(target_names[i]);
 }
 
@@ -248,7 +276,7 @@ RenderSettings::set_target(synfig::String name)
 {
 	target_name=name;
 	//TODO: Replace this condition
-	tparam_button->set_sensitive(target_name.compare("ffmpeg") && target_name.compare("png-spritesheet")?false:true);
+	tparam_button->set_sensitive(!(target_name.compare("ffmpeg") && target_name.compare("png-spritesheet")));
 }
 
 void
@@ -264,9 +292,9 @@ RenderSettings::on_targetparam_pressed()
 {
 	Dialog_TargetParam * dialogtp;
 	//TODO: Replace this conditions too
-	if (target_name.compare("ffmpeg") == 0)
+	if (!target_name.compare("ffmpeg"))
 		dialogtp = new Dialog_FFmpegParam (*this);
-	else if (target_name.compare("png-spritesheet") == 0)
+	else if (!target_name.compare("png-spritesheet"))
 		dialogtp = new Dialog_SpriteSheetParam (*this);
 	else
 		return;
@@ -311,7 +339,7 @@ RenderSettings::on_render_pressed()
 				synfig::info("unknown extension");
 			}
 		}
-		catch(std::runtime_error x)
+		catch(std::runtime_error& x)
 		{
 			canvas_interface_->get_ui_interface()->error(_("Unable to determine proper target from filename."));
 			return;
@@ -325,38 +353,43 @@ RenderSettings::on_render_pressed()
 	}
 
 	hide();
-	
+
 	render_passes.clear();
 	if (toggle_extract_alpha.get_active())
 	{
 		String filename_alpha(filename_sans_extension(filename)+"-alpha"+filename_extension(filename));
-		
+
 		render_passes.push_back(make_pair(TARGET_ALPHA_MODE_EXTRACT, filename_alpha));
 		render_passes.push_back(make_pair(TARGET_ALPHA_MODE_REDUCE, filename));
-		
+
 	} else {
 		render_passes.push_back(make_pair(TARGET_ALPHA_MODE_KEEP, filename));
 	}
 	
-	submit_next_render_pass();
+	App::dock_info_->set_n_passes_requested(render_passes.size());
+	App::dock_info_->set_n_passes_pending(render_passes.size());
+	App::dock_info_->set_render_progress(0.0);
+	App::dock_manager->find_dockable("info").present(); //Bring Dock_Info to front
 	
+	progress_logger->clear();
+	submit_next_render_pass();
+
 	return;
 }
-	
+
 void
 RenderSettings::submit_next_render_pass()
 {
 	if (render_passes.size()>0) {
 		pair<TargetAlphaMode,String> pass_info = render_passes.back();
 		render_passes.pop_back();
+
+		App::dock_info_->set_n_passes_pending(render_passes.size()); //! Decrease until 0
+		App::dock_info_->set_render_progress(0.0); //For this pass
 		
 		TargetAlphaMode pass_alpha_mode = pass_info.first;
-#ifdef WIN32
-		String pass_filename = Glib::locale_from_utf8(pass_info.second);
-#else
 		String pass_filename = pass_info.second;
-#endif
-		
+
 		Target::Handle target=Target::create(calculated_target_name,pass_filename, tparam);
 		if(!target)
 		{
@@ -378,7 +411,7 @@ RenderSettings::submit_next_render_pass()
 		// If we are to only render the current frame
 		if(toggle_single_frame.get_active())
 			rend_desc.set_time(canvas_interface_->get_time());
-	
+
 		target->set_rend_desc(&rend_desc);
 		target->set_quality((int)adjustment_quality->get_value());
 		if( !target->init(canvas_interface_->get_ui_interface().get()) ){
@@ -388,8 +421,8 @@ RenderSettings::submit_next_render_pass()
 		if(pass_alpha_mode!=TARGET_ALPHA_MODE_KEEP)
 			target->set_alpha_mode(pass_alpha_mode);
 
-		canvas_interface_->get_ui_interface()->task(_("Rendering ")+pass_filename);
-	
+		canvas_interface_->get_ui_interface()->task(strprintf(_("Rendering %s"), pass_filename.c_str()));
+
 		/*
 		if(async_renderer)
 		{
@@ -397,7 +430,7 @@ RenderSettings::submit_next_render_pass()
 			async_renderer.detach();
 		}
 		*/
-		async_renderer=new AsyncRenderer(target);
+		async_renderer=new AsyncRenderer(target, progress_logger.get());
 		async_renderer->signal_finished().connect( sigc::mem_fun(*this,&RenderSettings::on_finished));
 		async_renderer->start();
 		/*
@@ -417,16 +450,33 @@ RenderSettings::submit_next_render_pass()
 }
 
 void
-RenderSettings::on_finished()
+RenderSettings::on_finished(std::string error_message)
 {
-	String text(_("File rendered successfully"));
+	String text(_("Animation rendered successfully"));
 	Real execution_time = async_renderer ? async_renderer->get_execution_time() : 0.0;
 	if (execution_time > 0) text += strprintf(" (%f %s)", execution_time, _("sec"));
 
-	canvas_interface_->get_ui_interface()->task(text);
+	bool success = error_message.empty();
+
+	if (success)
+		canvas_interface_->get_ui_interface()->task(text);
 	canvas_interface_->get_ui_interface()->amount_complete(0,10000);
+
+	bool really_finished = (render_passes.size() == 0); //Must be checked BEFORE submit_next_render_pass();
 	
 	submit_next_render_pass();
+
+	if (really_finished) { // Because of multi-pass render
+		if (App::use_render_done_sound && App::sound_render_done) {
+			App::sound_render_done->set_position(Time());
+			App::sound_render_done->set_playing(true);
+		}
+		App::dock_info_->set_render_progress(1.0);
+	}
+
+	// Play the sound before show error dialog!
+	if (!success)
+		canvas_interface_->get_ui_interface()->error(error_message);
 }
 
 void
