@@ -1,13 +1,14 @@
 /* === S Y N F I G ========================================================= */
 /*!	\file canvastreestore.cpp
-**	\brief Template File
+**	\brief Canvas tree store
 **
 **	$Id$
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2008 Chris Moore
-**  Copyright (c) 2011 Carlos López
+**	Copyright (c) 2011 Carlos López
+**	Copyright (c) 2016 caryoscelus
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -31,6 +32,8 @@
 #	include <config.h>
 #endif
 
+#include <synfig/general.h>
+
 #include "trees/canvastreestore.h"
 #include <synfig/valuenode.h>
 #include "iconcontroller.h"
@@ -45,7 +48,7 @@
 #include <ETL/clock>
 #include <synfig/interpolation.h>
 
-#include "general.h"
+#include <gui/localization.h>
 
 #endif
 
@@ -102,6 +105,8 @@ CanvasTreeStore::get_value_vfunc(const Gtk::TreeModel::iterator& iter, int colum
 		Glib::Value<synfig::ValueBase> x;
 		g_value_init(x.gobj(),x.value_type());
 
+		ValueNode::Handle value_node = value_desc.get_value_node();
+		
 		if(!value_desc)
 		{
 			x.set(ValueBase());
@@ -110,13 +115,13 @@ CanvasTreeStore::get_value_vfunc(const Gtk::TreeModel::iterator& iter, int colum
 		if(value_desc.is_const())
 			x.set(value_desc.get_value());
 		else
-		if(value_desc.is_value_node())
+		if(value_node)
 		{
 			Type &type(value_desc.get_value_type());
 			if (type == type_bone_object)
 			{
 				Time time(canvas_interface()->get_time());
-				Bone bone((*(value_desc.get_value_node()))(time).get(Bone()));
+				Bone bone((*value_node)(time).get(Bone()));
 				String display(String(bone.get_name()));
 				ValueNode_Bone::ConstHandle parent(bone.get_parent());
 				if (!parent->is_root())
@@ -127,19 +132,19 @@ CanvasTreeStore::get_value_vfunc(const Gtk::TreeModel::iterator& iter, int colum
 			if (type == type_bone_weight_pair)
 			{
 				Time time(canvas_interface()->get_time());
-				BoneWeightPair bone_weight_pair((*(value_desc.get_value_node()))(time).get(BoneWeightPair()));
+				BoneWeightPair bone_weight_pair((*value_node)(time).get(BoneWeightPair()));
 				x.set(bone_weight_pair.get_string());
 			}
 			else
 			if (type == type_segment
-			 || type == type_list
-			 || type == type_bline_point)
+			|| type == type_list
+			|| type == type_bline_point)
 			{
 				x.set(value_desc.get_value_type().description.local_name);
 			}
 			else
 			{
-				x.set((*value_desc.get_value_node())(canvas_interface()->get_time()));
+				x.set((*value_node)(canvas_interface()->get_time()));
 			}
 		}
 		else
@@ -172,7 +177,7 @@ CanvasTreeStore::get_value_vfunc(const Gtk::TreeModel::iterator& iter, int colum
 		Glib::Value<bool> x;
 		g_value_init(x.gobj(),x.value_type());
 
-		x.set(value_desc.is_value_node() && value_desc.get_value_node()->rcount()>1);
+		x.set(value_desc.is_value_node() && value_desc.get_value_node() && value_desc.get_value_node()->rcount()>1);
 
 		g_value_init(value.gobj(),x.value_type());
 		g_value_copy(x.gobj(),value.gobj());
@@ -229,7 +234,18 @@ CanvasTreeStore::get_value_vfunc(const Gtk::TreeModel::iterator& iter, int colum
 		Glib::Value<bool> x;
 		g_value_init(x.gobj(),x.value_type());
 
-		x.set(!value_desc.is_value_node() || synfigapp::is_editable(value_desc.get_value_node()));
+		x.set(
+			// Temporary fix crash when trying to edit bline point
+			// https://github.com/synfig/synfig/issues/264
+			// TODO: move this check into a more proper place
+				value_desc.get_value_type() != type_bline_point
+			&&	value_desc.get_value_type() != type_width_point
+			&&	value_desc.get_value_type() != type_dash_item
+			&&	(
+				!value_desc.is_value_node()
+			||	synfigapp::is_editable(value_desc.get_value_node())
+			)
+		);
 
 		g_value_init(value.gobj(),x.value_type());
 		g_value_copy(x.gobj(),value.gobj());
@@ -252,8 +268,15 @@ CanvasTreeStore::get_value_vfunc(const Gtk::TreeModel::iterator& iter, int colum
 		else
 		{
 			stype=value_desc.get_value_type().description.local_name;
-			if(!value_desc.is_const())
-				stype+=" (" + value_desc.get_value_node()->get_local_name() + ")";
+			if(!value_desc.is_const()) {
+				const ValueNode::LooseHandle value_node = value_desc.get_value_node();
+				if (value_node) {
+					stype+=" (" + value_node->get_local_name() + ")";
+				} else {
+					stype+=" (> " + string(_("Invalid ValueNode")) + " <)";
+					synfig::error(_("Invalid ValueNode"));
+				}
+			}
 		}
 		x.set(stype.c_str());
 		g_value_init(value.gobj(),x.value_type());
@@ -289,12 +312,17 @@ CanvasTreeStore::get_value_vfunc(const Gtk::TreeModel::iterator& iter, int colum
 			ValueNode::Handle value_node=value_desc.get_value_node();
 
 			// Setup the row's label
-			if(value_node->get_id().empty())
-				x.set(Glib::ustring((*iter)[model.name]));
-			else if(Glib::ustring((*iter)[model.name]).empty())
-				x.set(value_node->get_id());
-			else
-				x.set(Glib::ustring((*iter)[model.name])+" ("+value_node->get_id()+')');
+			if (value_node) {
+				if(value_node->get_id().empty())
+					x.set(Glib::ustring((*iter)[model.name]));
+				else if(Glib::ustring((*iter)[model.name]).empty())
+					x.set(value_node->get_id());
+				else
+					x.set(Glib::ustring((*iter)[model.name])+" ("+value_node->get_id()+')');
+			} else {
+				x.set(" (> " + string(_("Invalid ValueNode")) + " <)");
+				synfig::error(_("Invalid ValueNode"));
+			}
 		}
 
 		g_value_init(value.gobj(),x.value_type());
@@ -543,7 +571,7 @@ CanvasTreeStore::set_row(Gtk::TreeRow row,synfigapp::ValueDesc value_desc, bool
 			return;
 		}
 	}
-	catch(synfig::Exception::IDNotFound x)
+	catch (const synfig::Exception::IDNotFound& x)
 	{
 		synfig::error(__FILE__":%d: IDNotFound thrown",__LINE__);
 		erase(row);
