@@ -1,12 +1,11 @@
 /* === S Y N F I G ========================================================= */
 /*!	\file layergrouptreestore.cpp
-**	\brief Template File
-**
-**	$Id$
+**	\brief Layer set tree model
 **
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2008 Chris Moore
+**	Copyright (c) 2017 caryoscelus
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -128,7 +127,7 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 	if(column==model.child_layers.index())
 	{
 		Glib::Value<LayerList> x;
-		g_value_init(x.gobj(),x.value_type());
+		x.init(x.value_type());
 
 		if((bool)(*iter)[model.is_group])
 		{
@@ -143,13 +142,13 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 			x.set(layer_list);
 		}
 
-		g_value_init(value.gobj(),x.value_type());
+		value.init(x.value_type());
 		value=x;
 	}
 	else if(column==model.all_layers.index())
 	{
 		Glib::Value<LayerList> x;
-		g_value_init(x.gobj(),x.value_type());
+		x.init(x.value_type());
 
 		if((bool)(*iter)[model.is_group])
 		{
@@ -158,7 +157,8 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 			for(;child_iter;++child_iter)
 			{
 				LayerList layer_list2((LayerList)(*child_iter)[model.all_layers]);
-				for(;layer_list2.size();layer_list2.pop_front())
+				//for(;layer_list2.size();layer_list2.pop_front())
+				for(;!layer_list2.empty();layer_list2.pop_front())
 					layer_list.push_back(layer_list2.front());
 			}
 			x.set(layer_list);
@@ -170,9 +170,25 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 			x.set(layer_list);
 		}
 
-		g_value_init(value.gobj(),x.value_type());
+		value.init(x.value_type());
 		value=x;
 	}
+	else if (column == model.z_depth.index())
+	{
+		Glib::Value<float> x;
+		x.init(x.value_type());
+		if ((bool)(*iter)[model.is_layer])
+		{
+			Layer::Handle layer = (Layer::Handle)(*iter)[model.layer];
+			x.set(layer->get_true_z_depth(canvas_interface()->get_time()));
+		}
+		else
+		{
+			x.set(0.0);
+		}
+		value.init(x.value_type());
+		value = x;
+	}
 	else if(column==model.group_name.index())
 	{
 		if((bool)(*iter)[model.is_group])
@@ -184,9 +200,9 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 		if(iter->parent())
 			return get_value_vfunc(iter->parent(),model.group_name.index(),value);
 		Glib::Value<Glib::ustring> x;
-		g_value_init(x.gobj(),x.value_type());
+		x.init(x.value_type());
 		x.set(Glib::ustring());
-		g_value_init(value.gobj(),x.value_type());
+		value.init(x.value_type());
 		value=x;
 	}
 	else if(column==model.label.index())
@@ -194,7 +210,7 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 		if((bool)(*iter)[model.is_group])
 		{
 			Glib::Value<Glib::ustring> x;
-			g_value_init(x.gobj(),x.value_type());
+			x.init(x.value_type());
 
 			Glib::ustring group_name((*iter)[model.group_name]);
 
@@ -204,8 +220,7 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 
 			x.set(group_name);
 
-			g_value_init(value.gobj(),x.value_type());
-
+			value.init(x.value_type());
 			value=x;
 		}
 		else if((bool)(*iter)[model.is_layer])
@@ -215,12 +230,11 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 			if(!layer)return;
 
 			Glib::Value<Glib::ustring> x;
-			g_value_init(x.gobj(),x.value_type());
+			x.init(x.value_type());
 
 			x.set(layer->get_non_empty_description());
 
-			g_value_init(value.gobj(),x.value_type());
-			//g_value_copy(x.gobj(),value.gobj());
+			value.init(x.value_type());
 			value=x;
 		}
 	}
@@ -232,13 +246,11 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 		if(!layer)return;
 
 		Glib::Value<Glib::ustring> x;
-		g_value_init(x.gobj(),x.value_type());
-
+		x.init(x.value_type());
 
 		x.set(layer->get_local_name());
 
-		g_value_init(value.gobj(),x.value_type());
-		//g_value_copy(x.gobj(),value.gobj());
+		value.init(x.value_type());
 		value=x;
 	}
 	else
@@ -249,20 +261,18 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 		if(!layer)return;
 
 		Glib::Value<Canvas::Handle> x;
-		g_value_init(x.gobj(),x.value_type());
-
+		x.init(x.value_type());
 
 		x.set(layer->get_canvas());
 
-		g_value_init(value.gobj(),x.value_type());
-		//g_value_copy(x.gobj(),value.gobj());
+		value.init(x.value_type());
 		value=x;
 	}
 	else
 	if(column==model.active.index())
 	{
 		Glib::Value<bool> x;
-		g_value_init(x.gobj(),x.value_type());
+		x.init(x.value_type());
 
 		if((bool)(*iter)[model.is_layer])
 		{
@@ -284,14 +294,14 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 		else
 			x.set(false);
 
-		g_value_init(value.gobj(),x.value_type());
+		value.init(x.value_type());
 		g_value_copy(x.gobj(),value.gobj());
 	}
 	else
 	if(column==model.icon.index())
 	{
 		Glib::Value<Glib::RefPtr<Gdk::Pixbuf> > x;
-		g_value_init(x.gobj(),x.value_type());
+		x.init(x.value_type());
 
 		if((bool)(*iter)[model.is_layer])
 		{
@@ -303,7 +313,7 @@ LayerGroupTreeStore::get_value_vfunc (const Gtk::TreeModel::iterator& iter, int
 		if((bool)(*iter)[model.is_group])
 			x.set(group_icon);
 
-		g_value_init(value.gobj(),x.value_type());
+		value.init(x.value_type());
 		g_value_copy(x.gobj(),value.gobj());
 	}
 	else
@@ -333,7 +343,7 @@ LayerGroupTreeStore::set_value_impl(const Gtk::TreeModel::iterator& iter, int co
 		if(column==model.label.index())
 		{
 			Glib::Value<Glib::ustring> x;
-			g_value_init(x.gobj(),model.label.type());
+			x.init(model.label.type());
 			g_value_copy(value.gobj(),x.gobj());
 
 			if((bool)(*iter)[model.is_layer])
@@ -412,7 +422,7 @@ LayerGroupTreeStore::set_value_impl(const Gtk::TreeModel::iterator& iter, int co
 		if(column==model.active.index())
 		{
 			Glib::Value<bool> x;
-			g_value_init(x.gobj(),model.active.type());
+			x.init(model.active.type());
 			g_value_copy(value.gobj(),x.gobj());
 
 			if((bool)(*iter)[model.is_layer])
@@ -455,7 +465,7 @@ LayerGroupTreeStore::set_value_impl(const Gtk::TreeModel::iterator& iter, int co
 			Gtk::TreeStore::set_value_impl(iter,column, value);
 
 	}
-	catch(std::exception x)
+	catch(std::exception& x)
 	{
 		g_warning("%s", x.what());
 	}
@@ -707,13 +717,13 @@ LayerGroupTreeStore::rebuild()
 		clear();
 		Canvas::Handle canvas(canvas_interface()->get_canvas());
 		std::set<String> groups(canvas->get_groups());
-		for(;groups.size();groups.erase(groups.begin()))
+		for(;!groups.empty();groups.erase(groups.begin()))
 		{
 			String group(*groups.begin());
 			Gtk::TreeRow row(on_group_added(group));
 			std::set<Layer::Handle> layers(canvas->get_layers_in_group(group));
 
-			for(;layers.size();layers.erase(layers.begin()))
+			for(;!layers.empty();layers.erase(layers.begin()))
 			{
 				Gtk::TreeRow layer_row(*(prepend(row.children())));
 				Layer::Handle layer(*layers.begin());
@@ -733,6 +743,7 @@ LayerGroupTreeStore::rebuild()
 		throw;
 	}
 	rebuilding=false;
+	resort();
 	// synfig::info("LayerGroupTreeStore::rebuild() took %f seconds",float(timer()));
 }
 
@@ -742,6 +753,21 @@ LayerGroupTreeStore::refresh()
 	rebuild();
 }
 
+void
+LayerGroupTreeStore::resort()
+{
+	// For some reason Gtk doesn't seem to have a method that does just that and
+	// ignores calls to set_sort_column if sorting params are unchanged, so we
+	// have to call it twice
+	int sort_column;
+	Gtk::SortType sort_order;
+	if (get_sort_column_id(sort_column, sort_order)) {
+		Gtk::SortType reverse_order = sort_order == Gtk::SORT_DESCENDING ? Gtk::SORT_ASCENDING : Gtk::SORT_DESCENDING;
+		set_sort_column(sort_column, reverse_order);
+		set_sort_column(sort_column, sort_order);
+	}
+}
+
 void
 LayerGroupTreeStore::refresh_row(Gtk::TreeModel::Row &row)
 {
@@ -853,6 +879,9 @@ LayerGroupTreeStore::on_group_pair_added(synfig::String group, etl::handle<synfi
 
 	Gtk::TreeRow layer_row(*(append(iter->children())));
 	set_row_layer(layer_row,layer);
+
+	resort();
+
 	on_activity();
 }
 
@@ -923,6 +952,7 @@ LayerGroupTreeStore::on_layer_new_description(synfig::Layer::Handle handle,synfi
 		else
 			//row[model.label]=layer->get_description();
 			row[model.tooltip]=layer->get_local_name();
+		resort();
 	}
 	else
 	{
