@@ -38,6 +38,8 @@
 #include "cellrenderer/cellrenderer_value.h"
 #include "cellrenderer/cellrenderer_timetrack.h"
 #include <synfigapp/action.h>
+#include <synfigapp/action_system.h>
+#include <synfigapp/actions/layerremove.h>
 #include <synfigapp/instance.h>
 #include <gtkmm/scrolledwindow.h>
 #include <gtkmm/paned.h>
@@ -51,6 +53,8 @@
 
 #include <gui/localization.h>
 
+#include <gui/exception_guard.h>
+
 #endif
 
 /* === U S I N G =========================================================== */
@@ -74,45 +78,71 @@ using namespace studio;
 	button->show()
 #endif
 
-#ifndef NORMAL_BUTTON
-#define NORMAL_BUTTON(button,stockid,tooltip)	\
-	button = manage(new class Gtk::Button());	\
-	icon=manage(new Gtk::Image(Gtk::StockID(stockid),Gtk::ICON_SIZE_BUTTON));	\
-	button->add(*icon);	\
-	button->set_tooltip_text(tooltip);	\
-	icon->set_padding(0,0);\
-	icon->show();	\
-	/*button->set_relief(Gtk::RELIEF_NONE);*/ \
-	button->show()
-#endif
-
-#define NEW_SMALL_BUTTON(x,y,z)	Gtk::Button *SMALL_BUTTON(x,y,z)
-
-#define NOT_IMPLEMENTED_SLOT sigc::mem_fun(*reinterpret_cast<studio::CanvasViewUIInterface*>(get_ui_interface().get()),&studio::CanvasViewUIInterface::not_implemented)
-
 /* === G L O B A L S ======================================================= */
 
 /* === P R O C E D U R E S ================================================= */
 
 /* === M E T H O D S ======================================================= */
 
+/*
+	Return true if we process event,
+	False to pass it
+*/
+bool LayerTree::on_key_press_event(GdkEventKey* event)
+{
+	SYNFIG_EXCEPTION_GUARD_BEGIN()
+	switch (event->keyval) {
+		case GDK_KEY_Delete: {
+			LayerList layers = get_selected_layers();
+
+			if (layers.size() == 0) return true; // nothing to do
+			
+
+			synfigapp::Action::Handle action(synfigapp::Action::LayerRemove::create());
+			assert(action);
+			if (!action)
+				return true;
+			
+			action->set_param("canvas", layer_tree_store_->canvas_interface()->get_canvas());
+			action->set_param("canvas_interface", layer_tree_store_->canvas_interface());
+
+			for(LayerList::const_iterator i = layers.begin(); i != layers.end(); ++i)
+				action->set_param("layer", *i);
+
+			layer_tree_store_->canvas_interface()->get_instance()->perform_action(action);
+			return true;
+
+		} 
+		case GDK_KEY_F2: {
+			Glib::RefPtr<Gtk::TreeSelection> selection = get_selection();
+			std::vector<Gtk::TreeModel::Path> pathList = selection->get_selected_rows();
+			if (pathList.size() != 1) return true;
+
+			const Gtk::TreeModel::Path& path = pathList.front();
+			Gtk::TreeView::Column& focus_column = *(layer_tree_view().get_column(2));
+
+			layer_tree_view().set_cursor(
+				path, 
+				focus_column, 
+				true
+			);
+			return true;
+		}
+	}
+
+    return false;
+	SYNFIG_EXCEPTION_GUARD_END_BOOL(true)
+}
+
+
 LayerTree::LayerTree():
 	layer_amount_adjustment_(Gtk::Adjustment::create(1,0,1,0.01,0.01,0))
 {
-	param_tree_view_=new Gtk::TreeView;
-	layer_tree_view_=new Gtk::TreeView;
-
-	//Gtk::HPaned* hpaned(manage(new Gtk::HPaned()));
-	//hpaned->show();
-	//attach(*hpaned, 0, 3, 0, 1, Gtk::EXPAND|Gtk::FILL,Gtk::EXPAND|Gtk::FILL, 0, 0);
-	//attach(*create_layer_tree(), 0, 3, 0, 1, Gtk::EXPAND|Gtk::FILL,Gtk::EXPAND|Gtk::FILL, 0, 0);
+	layer_tree_view().signal_key_press_event().connect(sigc::mem_fun(*this, &LayerTree::on_key_press_event));
 
 	create_layer_tree();
 	create_param_tree();
 
-	//hpaned->pack1(*create_layer_tree(),false,false);
-	//hpaned->pack2(*create_param_tree(),true,false);
-	//hpaned->set_position(200);
 	hbox=manage(new Gtk::HBox());
 
 	attach(*hbox, 0, 1, 1, 2, Gtk::FILL|Gtk::SHRINK, Gtk::SHRINK, 0, 0);
@@ -126,7 +156,6 @@ LayerTree::LayerTree():
 	layer_amount_adjustment_->signal_value_changed().connect(sigc::mem_fun(*this, &studio::LayerTree::on_amount_value_changed));
 
 	Gtk::Image *icon;
-	//Gtk::IconSize iconsize(Gtk::IconSize::from_name("synfig-small_icon"));
 	Gtk::IconSize iconsize(Gtk::ICON_SIZE_SMALL_TOOLBAR);
 
 	SMALL_BUTTON(button_raise,"gtk-go-up","Raise");
@@ -155,18 +184,18 @@ LayerTree::LayerTree():
 
 	get_selection()->signal_changed().connect(sigc::mem_fun(*this, &studio::LayerTree::on_selection_changed));
 
-	get_layer_tree_view().set_reorderable(true);
+	layer_tree_view().set_reorderable(true);
 	get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
-	//get_param_tree_view().get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
-	get_layer_tree_view().show();
-	get_param_tree_view().show();
+	//param_tree_view().get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
+	layer_tree_view().show();
+	param_tree_view().show();
 
 	hbox->show();
 	layer_amount_hscale->show();
 	blend_method_widget.show();
 
-	get_param_tree_view().set_has_tooltip();
-	get_layer_tree_view().set_has_tooltip();
+	param_tree_view().set_has_tooltip();
+	layer_tree_view().set_has_tooltip();
 
 	disable_amount_changed_signal=false;
 
@@ -184,11 +213,9 @@ LayerTree::~LayerTree()
 		synfig::info("LayerTree::~LayerTree(): Deleted");
 }
 
-Gtk::Widget*
+void
 LayerTree::create_layer_tree()
 {
-	const LayerTreeStore::Model model;
-
 	{	// --- O N / O F F ----------------------------------------------------
 		Gtk::TreeView::Column* column = Gtk::manage( new Gtk::TreeView::Column(_(" ")) );
 
@@ -197,15 +224,15 @@ LayerTree::create_layer_tree()
 		cellrenderer->signal_toggled().connect(sigc::mem_fun(*this, &studio::LayerTree::on_layer_toggle));
 		column->pack_start(*cellrenderer,false);
 		column->add_attribute(cellrenderer->property_active(), layer_model.active);
-		get_layer_tree_view().append_column(*column);
+		layer_tree_view().append_column(*column);
 	}
 
 	{	// --- I C O N --------------------------------------------------------
 		int index;
 		// Set up the icon cell-renderer
-		index=get_layer_tree_view().append_column(_("Icon"),layer_model.icon);
-		Gtk::TreeView::Column* column = get_layer_tree_view().get_column(index-1);
-		get_layer_tree_view().set_expander_column(*column);
+		index=layer_tree_view().append_column(_("Icon"),layer_model.icon);
+		Gtk::TreeView::Column* column = layer_tree_view().get_column(index-1);
+		layer_tree_view().set_expander_column(*column);
 	}
 	{	// --- N A M E --------------------------------------------------------
 		Gtk::TreeView::Column* column = Gtk::manage( new Gtk::TreeView::Column(_("Name")) );
@@ -226,14 +253,14 @@ LayerTree::create_layer_tree()
 		column->set_clickable(true);
 		column->set_sort_column(layer_model.label);
 
-		get_layer_tree_view().append_column(*column);
+		layer_tree_view().append_column(*column);
 	}
 	{	// --- Z D E P T H ----------------------------------------------------
 		int index;
-		index=get_layer_tree_view().append_column(_("Z Depth"),layer_model.z_depth);
+		index=layer_tree_view().append_column(_("Z Depth"),layer_model.z_depth);
 		// Set up the Z-Depth label cell-renderer
 
-		column_z_depth=get_layer_tree_view().get_column(index-1);
+		column_z_depth=layer_tree_view().get_column(index-1);
 		column_z_depth->set_reorderable();
 		column_z_depth->set_resizable();
 		column_z_depth->set_clickable();
@@ -241,35 +268,26 @@ LayerTree::create_layer_tree()
 		column_z_depth->set_sort_column(layer_model.z_depth);
 	}
 
-	get_layer_tree_view().set_enable_search(true);
-	get_layer_tree_view().set_search_column(layer_model.label);
-	get_layer_tree_view().set_search_equal_func(sigc::ptr_fun(&studio::LayerTreeStore::search_func));
+	layer_tree_view().set_enable_search(true);
+	layer_tree_view().set_search_column(layer_model.label);
+	layer_tree_view().set_search_equal_func(sigc::ptr_fun(&studio::LayerTreeStore::search_func));
 
 	std::vector<Gtk::TargetEntry> listTargets;
 	listTargets.push_back( Gtk::TargetEntry("LAYER") );
-	get_layer_tree_view().drag_dest_set(listTargets);
+	layer_tree_view().drag_dest_set(listTargets);
 
 	// This makes things easier to read.
-	get_layer_tree_view().set_rules_hint();
+	layer_tree_view().set_rules_hint();
 
 	// Make us more sensitive to several events
-	//get_layer_tree_view().add_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON1_MOTION_MASK | Gdk::BUTTON2_MOTION_MASK|Gdk::POINTER_MOTION_MASK);
+	//layer_tree_view().add_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON1_MOTION_MASK | Gdk::BUTTON2_MOTION_MASK|Gdk::POINTER_MOTION_MASK);
 
-	get_layer_tree_view().signal_event().connect(sigc::mem_fun(*this, &studio::LayerTree::on_layer_tree_event));
-	get_layer_tree_view().signal_query_tooltip().connect(sigc::mem_fun(*this, &studio::LayerTree::on_layer_tree_view_query_tooltip));
-	get_layer_tree_view().show();
-
-	Gtk::ScrolledWindow *scroll = manage(new class Gtk::ScrolledWindow());
-	scroll->set_can_focus(true);
-	scroll->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
-	//scroll->add(get_layer_tree_view());
-	scroll->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
-	scroll->show();
-
-	return scroll;
+	layer_tree_view().signal_event().connect(sigc::mem_fun(*this, &studio::LayerTree::on_layer_tree_event));
+	layer_tree_view().signal_query_tooltip().connect(sigc::mem_fun(*this, &studio::LayerTree::on_layer_tree_view_query_tooltip));
+	layer_tree_view().show();
 }
 
-Gtk::Widget*
+void
 LayerTree::create_param_tree()
 {
 	//Text attributes must be the same that TimeTrackView tree's to have aligned rows
@@ -307,9 +325,6 @@ LayerTree::create_param_tree()
 		text_cellrenderer->property_foreground()=Glib::ustring("#7f7f7f");
 		column->add_attribute(text_cellrenderer->property_foreground_set(),param_model.is_inconsistent);
 
-		// Pack the label into the column
-		//column->pack_start(param_model.label,true);
-
 		// Set up the value-node icon cell-renderer to be on the far right
 		Gtk::CellRendererPixbuf* valuenode_icon_cellrenderer = Gtk::manage( new Gtk::CellRendererPixbuf() );
 		column->pack_end(*valuenode_icon_cellrenderer,false);
@@ -325,7 +340,7 @@ LayerTree::create_param_tree()
 		column->set_clickable();
 		column->set_sort_column(param_model.name);
 
-		get_param_tree_view().append_column(*column);
+		param_tree_view().append_column(*column);
 	}
 	{	// --- V A L U E  -----------------------------------------------------
 		Gtk::TreeView::Column* column = Gtk::manage( new Gtk::TreeView::Column(_("Value")) );
@@ -342,7 +357,7 @@ LayerTree::create_param_tree()
 		cellrenderer_value->property_attributes()=attr_list;
 
 		// Finish setting up the column
-		get_param_tree_view().append_column(*column);
+		param_tree_view().append_column(*column);
 		column->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
 		column->set_fixed_width(150);
 		column->set_min_width(75);
@@ -367,7 +382,7 @@ LayerTree::create_param_tree()
 		column->add_attribute(static_icon_cellrenderer->property_visible(), param_model.is_static);
 
 		// Finish setting up the column
-		get_param_tree_view().append_column(*column);
+		param_tree_view().append_column(*column);
 		column->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
 		column->set_fixed_width(26);
 		column->set_min_width(26);
@@ -382,7 +397,7 @@ LayerTree::create_param_tree()
 		column->add_attribute(text_cellrenderer->property_text(), param_model.type);
 		text_cellrenderer->property_attributes()=attr_list;
 
-		get_param_tree_view().append_column(*column);
+		param_tree_view().append_column(*column);
 		column->set_reorderable();
 		column->set_resizable();
 		column->set_clickable();
@@ -402,8 +417,9 @@ LayerTree::create_param_tree()
 		cellrenderer_time_track->signal_waypoint_clicked_cellrenderer().connect(sigc::mem_fun(*this, &studio::LayerTree::on_waypoint_clicked_layertree) );
 		cellrenderer_time_track->signal_waypoint_changed().connect(sigc::mem_fun(*this, &studio::LayerTree::on_waypoint_changed) );
 		column->add_attribute(cellrenderer_time_track->property_value_desc(), param_model.value_desc);
-		column->add_attribute(cellrenderer_time_track->property_canvas(), param_model.canvas);
-		column->add_attribute(cellrenderer_time_track->property_visible(), param_model.is_value_node);
+		// this already added in constructor
+		//column->add_attribute(cellrenderer_time_track->property_canvas(), param_model.canvas);
+	    column->add_attribute(cellrenderer_time_track->property_visible(), param_model.is_value_node);
 
 		// Finish setting up the column
 		column->set_reorderable();
@@ -413,48 +429,39 @@ LayerTree::create_param_tree()
 		column->set_resizable();
 
 		if (!getenv("SYNFIG_DISABLE_PARAMS_PANEL_TIMETRACK"))
-			get_param_tree_view().append_column(*column);
+			param_tree_view().append_column(*column);
 	}
 #endif	// TIMETRACK_IN_PARAMS_PANEL
 
 	// This makes things easier to read.
-	get_param_tree_view().set_rules_hint();
+	param_tree_view().set_rules_hint();
 
 	// Make us more sensitive to several events
-	get_param_tree_view().add_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON1_MOTION_MASK | Gdk::BUTTON2_MOTION_MASK|Gdk::POINTER_MOTION_MASK);
+	param_tree_view().add_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON1_MOTION_MASK | Gdk::BUTTON2_MOTION_MASK|Gdk::POINTER_MOTION_MASK);
 
-	get_param_tree_view().signal_event().connect(sigc::mem_fun(*this, &studio::LayerTree::on_param_tree_event));
-	get_param_tree_view().signal_query_tooltip().connect(sigc::mem_fun(*this, &studio::LayerTree::on_param_tree_view_query_tooltip));
+	param_tree_view().signal_event().connect(sigc::mem_fun(*this, &studio::LayerTree::on_param_tree_event));
+	param_tree_view().signal_query_tooltip().connect(sigc::mem_fun(*this, &studio::LayerTree::on_param_tree_view_query_tooltip));
 	// Column widget label event used to retrieve column size
-	Gtk::Widget* columnzero_label = get_param_tree_view().get_column(0)->get_widget ();
+	Gtk::Widget* columnzero_label = param_tree_view().get_column(0)->get_widget ();
 	columnzero_label->signal_style_updated().connect(sigc::mem_fun(*this, &studio::LayerTree::on_param_column_label_tree_style_updated));
 	columnzero_label->signal_draw().connect(sigc::mem_fun(*this, &studio::LayerTree::on_param_column_label_tree_draw));
-	get_param_tree_view().show();
-
-	Gtk::ScrolledWindow *scroll = manage(new class Gtk::ScrolledWindow());
-	scroll->set_can_focus(true);
-	scroll->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
-	//scroll->add(get_param_tree_view());
-	scroll->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
-	scroll->show();
+	param_tree_view().show();
 
 	// To get the initial style
 	param_tree_style_changed = true;
 	param_tree_header_height = 0;
 
 	//column_time_track->set_visible(false);
-
-	return scroll;
 }
 
 void
-LayerTree::on_waypoint_changed( synfig::Waypoint waypoint , synfig::ValueNode::Handle value_node)
+LayerTree::on_waypoint_changed(synfig::Waypoint& waypoint , synfig::ValueNode::Handle value_node)
 {
 	synfigapp::Action::ParamList param_list;
 	param_list.add("canvas",layer_tree_store_->canvas_interface()->get_canvas());
 	param_list.add("canvas_interface",layer_tree_store_->canvas_interface());
 	param_list.add("value_node",value_node);
-	param_list.add("waypoint",waypoint);
+	param_list.add("waypoint", waypoint);
 //	param_list.add("time",canvas_interface()->get_time());
 
 	etl::handle<studio::Instance>::cast_static(layer_tree_store_->canvas_interface()->get_instance())->process_action("WaypointSetSmart", param_list);
@@ -476,20 +483,20 @@ LayerTree::select_layer(synfig::Layer::Handle layer)
 			path=Gtk::TreePath(iter);
 			for(j=i;j;j--)
 				path.up();
-			get_layer_tree_view().expand_row(path,false);
+			layer_tree_view().expand_row(path,false);
 		}
-		get_layer_tree_view().scroll_to_row(Gtk::TreePath(iter));
-		get_layer_tree_view().get_selection()->select(iter);
+		layer_tree_view().scroll_to_row(Gtk::TreePath(iter));
+		layer_tree_view().get_selection()->select(iter);
 	}
 }
 
 void
 LayerTree::select_all_children(Gtk::TreeModel::Children::iterator iter)
 {
-	get_layer_tree_view().get_selection()->select(iter);
+	layer_tree_view().get_selection()->select(iter);
 	if((bool)(*iter)[layer_model.children_lock])
 		return;
-	get_layer_tree_view().expand_row(layer_tree_store_->get_path(iter),false);
+	layer_tree_view().expand_row(layer_tree_store_->get_path(iter),false);
 	Gtk::TreeModel::Children children(iter->children());
 	for(iter=children.begin();iter!=children.end();++iter)
 		select_all_children(iter);
@@ -523,7 +530,7 @@ static inline void __layer_grabber(const Gtk::TreeModel::iterator& iter, LayerTr
 LayerTree::LayerList
 LayerTree::get_selected_layers()const
 {
-	Glib::RefPtr<Gtk::TreeSelection> selection=const_cast<Gtk::TreeView&>(get_layer_tree_view()).get_selection();
+	Glib::RefPtr<Gtk::TreeSelection> selection=const_cast<Gtk::TreeView&>(layer_tree_view()).get_selection();
 
 	if(!selection)
 		return LayerList();
@@ -556,7 +563,7 @@ LayerTree::get_selected_layer()const
 void
 LayerTree::clear_selected_layers()
 {
-	get_layer_tree_view().get_selection()->unselect_all();
+	layer_tree_view().get_selection()->unselect_all();
 }
 
 void
@@ -570,7 +577,7 @@ LayerTree::expand_layer(synfig::Layer::Handle layer)
 			iter=sorted_layer_tree_store_->convert_child_iter_to_iter(iter);
 
 		Gtk::TreePath path(iter);
-		get_layer_tree_view().expand_to_path(path);
+		layer_tree_view().expand_to_path(path);
 	}
 }
 
@@ -602,7 +609,7 @@ LayerTree::get_expanded_layers(LayerList &list, const Gtk::TreeNodeChildren &row
 			if(sorted_layer_tree_store_)
 				j = sorted_layer_tree_store_->convert_child_iter_to_iter(i);
 			Gtk::TreePath path(j);
-			if (const_cast<Gtk::TreeView*>(&get_layer_tree_view())->row_expanded(path))
+			if (const_cast<Gtk::TreeView*>(&layer_tree_view())->row_expanded(path))
 			{
 				list.push_back( (Layer::Handle)(*i)[model.layer] );
 				get_expanded_layers(list, i->children());
@@ -622,7 +629,7 @@ LayerTree::set_show_timetrack(bool x)
 void
 LayerTree::select_param(const synfigapp::ValueDesc& valuedesc)
 {
-    get_param_tree_view().get_selection()->unselect_all();
+    param_tree_view().get_selection()->unselect_all();
 
     Gtk::TreeIter iter;
     if(param_tree_store_->find_value_desc(valuedesc, iter))
@@ -634,11 +641,11 @@ LayerTree::select_param(const synfigapp::ValueDesc& valuedesc)
             path=Gtk::TreePath(iter);
             for(j=i;j;j--)
                 path.up();
-            get_param_tree_view().expand_row(path,false);
+            param_tree_view().expand_row(path,false);
         }
 
-        get_param_tree_view().scroll_to_row(Gtk::TreePath(iter));
-        get_param_tree_view().get_selection()->select(iter);
+        param_tree_view().scroll_to_row(Gtk::TreePath(iter));
+        param_tree_view().get_selection()->select(iter);
     }
 }
 
@@ -656,10 +663,10 @@ LayerTree::set_model(Glib::RefPtr<LayerTreeStore> layer_tree_store)
 		//sorted_store->set_sort_func(model.time.index(),sigc::mem_fun(&studio::KeyframeTreeStore::time_sorter));
 		//sorted_store->set_sort_column(model.time.index(), Gtk::SORT_ASCENDING);
 
-		get_layer_tree_view().set_model(sorted_layer_tree_store_);
+		layer_tree_view().set_model(sorted_layer_tree_store_);
 	}
 	else
-		get_layer_tree_view().set_model(layer_tree_store_);
+		layer_tree_view().set_model(layer_tree_store_);
 
 	layer_tree_store_->canvas_interface()->signal_dirty_preview().connect(sigc::mem_fun(*this,&studio::LayerTree::on_dirty_preview));
 
@@ -667,14 +674,14 @@ LayerTree::set_model(Glib::RefPtr<LayerTreeStore> layer_tree_store)
 
 	layer_tree_store_->canvas_interface()->signal_time_changed().connect(
 		sigc::mem_fun(
-			&get_param_tree_view(),
+			&param_tree_view(),
 			&Gtk::Widget::queue_draw
 		)
 	);
 	if(!param_tree_store_)
 	{
 		param_tree_store_=LayerParamTreeStore::create(layer_tree_store_->canvas_interface(), this);
-		get_param_tree_view().set_model(param_tree_store_);
+		param_tree_view().set_model(param_tree_store_);
 	}
 
 #ifdef TIMETRACK_IN_PARAMS_PANEL
@@ -684,13 +691,12 @@ LayerTree::set_model(Glib::RefPtr<LayerTreeStore> layer_tree_store)
 }
 
 void
-LayerTree::set_time_adjustment(const Glib::RefPtr<Gtk::Adjustment> &adjustment)
+LayerTree::set_time_model(const etl::handle<TimeModel> &x)
 {
-#ifdef TIMETRACK_IN_PARAMS_PANEL
-	cellrenderer_time_track->set_adjustment(adjustment);
-#endif	// TIMETRACK_IN_PARAMS_PANEL
-	adjustment->signal_value_changed().connect(sigc::mem_fun(get_param_tree_view(),&Gtk::TreeView::queue_draw));
-	adjustment->signal_changed().connect(sigc::mem_fun(get_param_tree_view(),&Gtk::TreeView::queue_draw));
+	#ifdef TIMETRACK_IN_PARAMS_PANEL
+	cellrenderer_time_track->set_time_model(x);
+	#endif
+	x->signal_time_changed().connect(sigc::mem_fun(param_tree_view(),&Gtk::TreeView::queue_draw));
 }
 
 void
@@ -725,7 +731,7 @@ LayerTree::on_selection_changed()
 		if(layer_list.empty())
 		{
 			last_top_selected_layer=0;
-			layer_tree_view_->get_selection()->select(last_top_selected_path);
+			layer_tree_view().get_selection()->select(last_top_selected_path);
 			return;
 		}
 	}
@@ -734,7 +740,7 @@ LayerTree::on_selection_changed()
 		if(!layer_list.empty())
 		{
 			last_top_selected_layer=layer_list.front();
-			last_top_selected_path=*layer_tree_view_->get_selection()->get_selected_rows().begin();
+			last_top_selected_path=layer_tree_view().get_selection()->get_selected_rows().front();
 		}
 		else
 		{
@@ -824,7 +830,7 @@ LayerTree::on_edited_value(const Glib::ustring&path_string,synfig::ValueBase val
 {
 	Gtk::TreePath path(path_string);
 
-	const Gtk::TreeRow row = *(get_param_tree_view().get_model()->get_iter(path));
+	const Gtk::TreeRow row = *(param_tree_view().get_model()->get_iter(path));
 	if(!row)
 		return;
 	row[param_model.value]=value;
@@ -836,11 +842,11 @@ LayerTree::on_layer_renamed(const Glib::ustring&path_string,const Glib::ustring&
 {
 	Gtk::TreePath path(path_string);
 
-	const Gtk::TreeRow row = *(get_layer_tree_view().get_model()->get_iter(path));
+	const Gtk::TreeRow row = *(layer_tree_view().get_model()->get_iter(path));
 	if(!row)
 		return;
 	row[layer_model.label]=value;
-	get_layer_tree_view().columns_autosize();
+	layer_tree_view().columns_autosize();
 }
 
 void
@@ -848,7 +854,7 @@ LayerTree::on_layer_toggle(const Glib::ustring& path_string)
 {
 	Gtk::TreePath path(path_string);
 
-	const Gtk::TreeRow row = *(get_layer_tree_view().get_model()->get_iter(path));
+	const Gtk::TreeRow row = *(layer_tree_view().get_model()->get_iter(path));
 	bool active=static_cast<bool>(row[layer_model.active]);
 	row[layer_model.active]=!active;
 }
@@ -884,6 +890,7 @@ LayerTree::on_waypoint_clicked_layertree(const etl::handle<synfig::Node>& node _
 bool
 LayerTree::on_layer_tree_event(GdkEvent *event)
 {
+	SYNFIG_EXCEPTION_GUARD_BEGIN()
     switch(event->type)
     {
 	case GDK_BUTTON_PRESS:
@@ -891,14 +898,15 @@ LayerTree::on_layer_tree_event(GdkEvent *event)
 			Gtk::TreeModel::Path path;
 			Gtk::TreeViewColumn *column;
 			int cell_x, cell_y;
-			if(!get_layer_tree_view().get_path_at_pos(
+			if(!layer_tree_view().get_path_at_pos(
 				int(event->button.x),int(event->button.y),	// x, y
 				path, // TreeModel::Path&
 				column, //TreeViewColumn*&
 				cell_x,cell_y //int&cell_x,int&cell_y
 				)
-			) break;
-			const Gtk::TreeRow row = *(get_layer_tree_view().get_model()->get_iter(path));
+			   )
+				return signal_no_layer_user_click()(reinterpret_cast<GdkEventButton*>(event));
+			const Gtk::TreeRow row = *(layer_tree_view().get_model()->get_iter(path));
 
 #ifdef TIMETRACK_IN_PARAMS_PANEL
 			if(column->get_first_cell()==cellrenderer_time_track)
@@ -918,7 +926,7 @@ LayerTree::on_layer_tree_event(GdkEvent *event)
 			Gtk::TreeModel::Path path;
 			Gtk::TreeViewColumn *column;
 			int cell_x, cell_y;
-			if(!get_layer_tree_view().get_path_at_pos(
+			if(!layer_tree_view().get_path_at_pos(
 				(int)event->button.x,(int)event->button.y,	// x, y
 				path, // TreeModel::Path&
 				column, //TreeViewColumn*&
@@ -926,10 +934,10 @@ LayerTree::on_layer_tree_event(GdkEvent *event)
 				)
 			) break;
 
-			if(!get_layer_tree_view().get_model()->get_iter(path))
+			if(!layer_tree_view().get_model()->get_iter(path))
 				break;
 
-			//Gtk::TreeRow row = *(get_layer_tree_view().get_model()->get_iter(path));
+			//Gtk::TreeRow row = *(layer_tree_view().get_model()->get_iter(path));
 
 #ifdef TIMETRACK_IN_PARAMS_PANEL
 			if(cellrenderer_time_track==column->get_first_cell())
@@ -956,11 +964,13 @@ LayerTree::on_layer_tree_event(GdkEvent *event)
 		break;
 	}
 	return false;
+	SYNFIG_EXCEPTION_GUARD_END_BOOL(true)
 }
 
 bool
 LayerTree::on_param_tree_event(GdkEvent *event)
 {
+	SYNFIG_EXCEPTION_GUARD_BEGIN()
     switch(event->type)
     {
 	case GDK_BUTTON_PRESS:
@@ -968,24 +978,24 @@ LayerTree::on_param_tree_event(GdkEvent *event)
 			Gtk::TreeModel::Path path;
 			Gtk::TreeViewColumn *column;
 			int cell_x, cell_y;
-			if(!get_param_tree_view().get_path_at_pos(
+			if(!param_tree_view().get_path_at_pos(
 				int(event->button.x),int(event->button.y),	// x, y
 				path, // TreeModel::Path&
 				column, //TreeViewColumn*&
 				cell_x,cell_y //int&cell_x,int&cell_y
 				)
 			) break;
-			const Gtk::TreeRow row = *(get_param_tree_view().get_model()->get_iter(path));
+			const Gtk::TreeRow row = *(param_tree_view().get_model()->get_iter(path));
 
 #ifdef TIMETRACK_IN_PARAMS_PANEL
 			if(column && column->get_first_cell()==cellrenderer_time_track)
 			{
 				Gdk::Rectangle rect;
-				get_param_tree_view().get_cell_area(path,*column,rect);
+				param_tree_view().get_cell_area(path,*column,rect);
 				cellrenderer_time_track->property_value_desc()=row[param_model.value_desc];
 				cellrenderer_time_track->property_canvas()=row[param_model.canvas];
 				cellrenderer_time_track->activate(event,*this,path.to_string(),rect,rect,Gtk::CellRendererState());
-				get_param_tree_view().queue_draw_area(rect.get_x(),rect.get_y(),rect.get_width(),rect.get_height());
+				param_tree_view().queue_draw_area(rect.get_x(),rect.get_y(),rect.get_width(),rect.get_height());
 				return true;
 				//return signal_param_user_click()(event->button.button,row,COLUMNID_TIME_TRACK);
 			}
@@ -1030,7 +1040,7 @@ LayerTree::on_param_tree_event(GdkEvent *event)
 			Gtk::TreeModel::Path path;
 			Gtk::TreeViewColumn *column;
 			int cell_x, cell_y;
-			if(!get_param_tree_view().get_path_at_pos(
+			if(!param_tree_view().get_path_at_pos(
 				(int)event->motion.x,(int)event->motion.y,	// x, y
 				path, // TreeModel::Path&
 				column, //TreeViewColumn*&
@@ -1038,21 +1048,21 @@ LayerTree::on_param_tree_event(GdkEvent *event)
 				)
 			) break;
 
-			if(!get_param_tree_view().get_model()->get_iter(path))
+			if(!param_tree_view().get_model()->get_iter(path))
 				break;
 
-			Gtk::TreeRow row = *(get_param_tree_view().get_model()->get_iter(path));
+			Gtk::TreeRow row = *(param_tree_view().get_model()->get_iter(path));
 
 #ifdef TIMETRACK_IN_PARAMS_PANEL
 			if(((event->motion.state&GDK_BUTTON1_MASK) || (event->motion.state&GDK_BUTTON3_MASK)) && column && cellrenderer_time_track==column->get_first_cell())
 			{
 				Gdk::Rectangle rect;
-				get_param_tree_view().get_cell_area(path,*column,rect);
+				param_tree_view().get_cell_area(path,*column,rect);
 				cellrenderer_time_track->property_value_desc()=row[param_model.value_desc];
 				cellrenderer_time_track->property_canvas()=row[param_model.canvas];
 				cellrenderer_time_track->activate(event,*this,path.to_string(),rect,rect,Gtk::CellRendererState());
-				get_param_tree_view().queue_draw();
-				//get_param_tree_view().queue_draw_area(rect.get_x(),rect.get_y(),rect.get_width(),rect.get_height());
+				param_tree_view().queue_draw();
+				//param_tree_view().queue_draw_area(rect.get_x(),rect.get_y(),rect.get_width(),rect.get_height());
 				return true;
 			}
 #endif	// TIMETRACK_IN_PARAMS_PANEL
@@ -1063,7 +1073,7 @@ LayerTree::on_param_tree_event(GdkEvent *event)
 			Gtk::TreeModel::Path path;
 			Gtk::TreeViewColumn *column;
 			int cell_x, cell_y;
-			if(!get_param_tree_view().get_path_at_pos(
+			if(!param_tree_view().get_path_at_pos(
 				(int)event->button.x,(int)event->button.y,	// x, y
 				path, // TreeModel::Path&
 				column, //TreeViewColumn*&
@@ -1071,21 +1081,21 @@ LayerTree::on_param_tree_event(GdkEvent *event)
 				)
 			) break;
 
-			if(!get_param_tree_view().get_model()->get_iter(path))
+			if(!param_tree_view().get_model()->get_iter(path))
 				break;
 
-			Gtk::TreeRow row = *(get_param_tree_view().get_model()->get_iter(path));
+			Gtk::TreeRow row = *(param_tree_view().get_model()->get_iter(path));
 
 #ifdef TIMETRACK_IN_PARAMS_PANEL
 			if(column && cellrenderer_time_track==column->get_first_cell())
 			{
 				Gdk::Rectangle rect;
-				get_param_tree_view().get_cell_area(path,*column,rect);
+				param_tree_view().get_cell_area(path,*column,rect);
 				cellrenderer_time_track->property_value_desc()=row[param_model.value_desc];
 				cellrenderer_time_track->property_canvas()=row[param_model.canvas];
 				cellrenderer_time_track->activate(event,*this,path.to_string(),rect,rect,Gtk::CellRendererState());
-				get_param_tree_view().queue_draw();
-				get_param_tree_view().queue_draw_area(rect.get_x(),rect.get_y(),rect.get_width(),rect.get_height());
+				param_tree_view().queue_draw();
+				param_tree_view().queue_draw_area(rect.get_x(),rect.get_y(),rect.get_width(),rect.get_height());
 				return true;
 
 			}
@@ -1096,6 +1106,7 @@ LayerTree::on_param_tree_event(GdkEvent *event)
 		break;
 	}
 	return false;
+	SYNFIG_EXCEPTION_GUARD_END_BOOL(true)
 }
 
 
@@ -1108,10 +1119,10 @@ LayerTree::on_param_tree_view_query_tooltip(int x, int y, bool keyboard_tooltip,
 	Gtk::TreeViewColumn *column;
 	int cell_x, cell_y;
 	int bx, by;
-	get_param_tree_view().convert_widget_to_bin_window_coords(x, y, bx, by);
-	if(!get_param_tree_view().get_path_at_pos(bx, by, path, column, cell_x,cell_y))
+	param_tree_view().convert_widget_to_bin_window_coords(x, y, bx, by);
+	if(!param_tree_view().get_path_at_pos(bx, by, path, column, cell_x,cell_y))
 		return false;
-	Gtk::TreeIter iter(get_param_tree_view().get_model()->get_iter(path));
+	Gtk::TreeIter iter(param_tree_view().get_model()->get_iter(path));
 	if(!iter)
 		return false;
 	Gtk::TreeRow row = *(iter);
@@ -1119,7 +1130,7 @@ LayerTree::on_param_tree_view_query_tooltip(int x, int y, bool keyboard_tooltip,
 	if(tooltip_string.empty())
 		return false;
 	tooltip->set_text(tooltip_string);
-	get_param_tree_view().set_tooltip_row(tooltip, path);
+	param_tree_view().set_tooltip_row(tooltip, path);
 	return true;
 }
 
@@ -1132,10 +1143,10 @@ LayerTree::on_layer_tree_view_query_tooltip(int x, int y, bool keyboard_tooltip,
 	Gtk::TreeViewColumn *column;
 	int cell_x, cell_y;
 	int bx, by;
-	get_layer_tree_view().convert_widget_to_bin_window_coords(x, y, bx, by);
-	if(!get_layer_tree_view().get_path_at_pos(bx, by, path, column, cell_x,cell_y))
+	layer_tree_view().convert_widget_to_bin_window_coords(x, y, bx, by);
+	if(!layer_tree_view().get_path_at_pos(bx, by, path, column, cell_x,cell_y))
 		return false;
-	Gtk::TreeIter iter(get_param_tree_view().get_model()->get_iter(path));
+	Gtk::TreeIter iter(param_tree_view().get_model()->get_iter(path));
 	if(!iter)
 		return false;
 	Gtk::TreeRow row = *(iter);
@@ -1143,7 +1154,7 @@ LayerTree::on_layer_tree_view_query_tooltip(int x, int y, bool keyboard_tooltip,
 	if(tooltip_string.empty())
 		return false;
 	tooltip->set_text(tooltip_string);
-	get_param_tree_view().set_tooltip_row(tooltip, path);
+	param_tree_view().set_tooltip_row(tooltip, path);
 	return true;
 }
 
@@ -1408,7 +1419,7 @@ bool
 LayerTree::update_param_tree_header_height()
 {
 	bool header_height_updated = false;
-	const Gtk::TreeViewColumn* column = get_param_tree_view().get_column (0);
+	const Gtk::TreeViewColumn* column = param_tree_view().get_column (0);
 	if (column)
 	{
 		if(column->get_widget())
