@@ -37,13 +37,16 @@
 #include <gtkmm/scale.h>
 #include <gtkmm/button.h>
 
-#include <synfigapp/canvasinterface.h>
-#include <synfigapp/value_desc.h>
-#include "layertreestore.h"
-#include "layerparamtreestore.h"
 #include <synfig/valuenodes/valuenode_animated.h>
 
-#include "widgets/widget_value.h"
+#include <synfigapp/value_desc.h>
+#include <synfigapp/canvasinterface.h>
+
+#include <widgets/widget_value.h>
+#include <gui/timemodel.h>
+
+#include "layertreestore.h"
+#include "layerparamtreestore.h"
 
 /* === M A C R O S ========================================================= */
 
@@ -101,10 +104,8 @@ public:
 	*/
 
 private:
-
-	Gtk::TreeView* layer_tree_view_;
-
-	Gtk::TreeView* param_tree_view_;
+	Gtk::TreeView layer_tree_view_;
+	Gtk::TreeView param_tree_view_;
 
 	Gtk::HBox *hbox;
 
@@ -135,6 +136,7 @@ private:
 	sigc::signal<void,synfigapp::ValueDesc,synfig::ValueBase> signal_edited_value_;
 
 	sigc::signal<bool, int, Gtk::TreeRow, ColumnID> signal_layer_user_click_;
+	sigc::signal<bool, GdkEventButton*> signal_no_layer_user_click_;
 
 	sigc::signal<bool, int, Gtk::TreeRow, ColumnID> signal_param_user_click_;
 
@@ -162,8 +164,8 @@ private:
 
 private:
 
-	Gtk::Widget* create_layer_tree();
-	Gtk::Widget* create_param_tree();
+	void create_layer_tree();
+	void create_param_tree();
 	//! Update the param_tree_view header height.
 	/*! \return true if param_tree_header_height updated, else false
 	*/
@@ -183,7 +185,7 @@ private:
 
 	void on_waypoint_clicked_layertree(const etl::handle<synfig::Node>& node, const synfig::Time&, const synfig::Time&, const synfig::Time&, int button);
 
-	void on_waypoint_changed( synfig::Waypoint waypoint , synfig::ValueNode::Handle value_node);
+	void on_waypoint_changed(synfig::Waypoint& waypoint , synfig::ValueNode::Handle value_node);
 
 	bool on_layer_tree_event(GdkEvent *event);
 
@@ -221,26 +223,23 @@ public:
 	*/
 
 public:
-
-	//Gtk::TreeView* get_param_tree_view() { return param_tree_view_; }
-	//Gtk::TreeView& param_tree_view() { return *param_tree_view_; }
 	Gtk::HBox& get_hbox() { return *hbox; }
 
-	Gtk::TreeView& get_layer_tree_view() { return *layer_tree_view_; }
-	Gtk::TreeView& get_param_tree_view() { return *param_tree_view_; }
+	Gtk::TreeView& layer_tree_view() { return layer_tree_view_; }
+	const Gtk::TreeView& layer_tree_view()const { return layer_tree_view_; }
 
-	const Gtk::TreeView& get_layer_tree_view()const { return *layer_tree_view_; }
-	const Gtk::TreeView& get_param_tree_view()const { return *param_tree_view_; }
+	Gtk::TreeView& param_tree_view() { return param_tree_view_; }
+	const Gtk::TreeView& param_tree_view()const { return param_tree_view_; }
 
-	Glib::RefPtr<Gtk::TreeSelection> get_selection() { return get_layer_tree_view().get_selection(); }
-	Glib::SignalProxy1< bool,GdkEvent* >  signal_event () { return get_layer_tree_view().signal_event(); }
+	Glib::RefPtr<Gtk::TreeSelection> get_selection() { return layer_tree_view().get_selection(); }
+	Glib::SignalProxy1< bool,GdkEvent* >  signal_event () { return layer_tree_view().signal_event(); }
 
 	LayerTree();
 	~LayerTree();
 
 	void set_model(Glib::RefPtr<LayerTreeStore> layer_tree_store_);
 
-	void set_time_adjustment(const Glib::RefPtr<Gtk::Adjustment> &adjustment);
+	void set_time_model(const etl::handle<TimeModel> &x);
 
 	void set_show_timetrack(bool x=true);
 
@@ -252,6 +251,8 @@ public:
 
 	sigc::signal<bool,int, Gtk::TreeRow, ColumnID>& signal_layer_user_click() { return signal_layer_user_click_; }
 
+	sigc::signal<bool, GdkEventButton*>& signal_no_layer_user_click() { return signal_no_layer_user_click_; }
+
 	sigc::signal<bool,int, Gtk::TreeRow, ColumnID>& signal_param_user_click() { return signal_param_user_click_; }
 
 	sigc::signal<void,synfigapp::ValueDesc,std::set<synfig::Waypoint,std::less<synfig::UniqueID> >,int>& signal_waypoint_clicked_layertree() { return signal_waypoint_clicked_layertree_; }
@@ -279,6 +280,8 @@ public:
 private:
 	void get_expanded_layers(LayerList &list, const Gtk::TreeNodeChildren &rows)const;
 
+	bool on_key_press_event(GdkEventKey* event);
+
 }; // END of LayerTree
 
 }; // END of namespace studio
