@@ -29,12 +29,15 @@
 #	include <config.h>
 #endif
 
+#include <synfig/general.h>
+#include <synfig/distance.h>
+
 #include <gtkmm/spinbutton.h>
 #include "widgets/widget_vector.h"
 #include "widgets/widget_distance.h"
 #include "app.h"
 
-#include "general.h"
+#include <gui/localization.h>
 
 #endif
 
@@ -55,17 +58,28 @@ using namespace studio;
 /* === M E T H O D S ======================================================= */
 
 Widget_Vector::Widget_Vector():
-	Gtk::HBox(false, 5),
-	x_adjustment(Gtk::Adjustment::create(0,-100000000,100000000,0.05,0.05,0)),
-	y_adjustment(Gtk::Adjustment::create(0,-100000000,100000000,0.05,0.05,0))
+	Glib::ObjectBase("widget_vector")
 {
-	Gtk::Label *label;
+	init();
+}
+
+Widget_Vector::Widget_Vector(BaseObjectType* cobject) :
+	Glib::ObjectBase("widget_vector"),
+	Gtk::Box(cobject)
+{
+	init();
+}
+
+void Widget_Vector::init() {
+	set_orientation(Gtk::ORIENTATION_HORIZONTAL);
+	set_homogeneous(false);
+	set_spacing(5);
+
+	x_adjustment = Gtk::Adjustment::create(0,-100000000,100000000,0.05,0.05,0);
+	y_adjustment = Gtk::Adjustment::create(0,-100000000,100000000,0.05,0.05,0);
+
 	int width_chars = 5;
 
-	label=manage(new class Gtk::Label("X:"));
-	label->set_alignment(0, 0.5);
-	//pack_start(*label, Gtk::PACK_SHRINK);
-	
 	entry_x=manage(new class Gtk::Entry());
 	entry_x->set_width_chars(width_chars);
 	entry_x->signal_changed().connect(sigc::mem_fun(*this,&studio::Widget_Vector::on_entry_x_changed));
@@ -84,11 +98,6 @@ Widget_Vector::Widget_Vector():
 	distance_x->signal_value_changed().connect(sigc::mem_fun(*this,&studio::Widget_Vector::on_value_changed));
 	pack_start(*distance_x, Gtk::PACK_EXPAND_WIDGET);
 
-	label=manage(new class Gtk::Label("Y:"));
-	label->set_alignment(0, 0.5);
-	label->show();
-	//pack_start(*label, Gtk::PACK_SHRINK);
-	
 	entry_y=manage(new class Gtk::Entry());
 	entry_y->set_width_chars(width_chars);
 	entry_y->signal_changed().connect(sigc::mem_fun(*this,&studio::Widget_Vector::on_entry_y_changed));
@@ -313,3 +322,27 @@ Widget_Vector::show_all_vfunc()
 	entry_y->show();
 	show();
 }
+
+GType Widget_Vector::gtype = 0;
+
+Glib::ObjectBase *
+Widget_Vector::wrap_new (GObject *o)
+{
+	if (gtk_widget_is_toplevel (GTK_WIDGET (o)))
+		return new Widget_Vector (GTK_BOX(o));
+	else
+		return Gtk::manage(new Widget_Vector(GTK_BOX(o)));
+}
+
+void
+Widget_Vector::register_type ()
+{
+	if (gtype)
+		return;
+
+	Widget_Vector dummy;
+
+	gtype = G_OBJECT_TYPE (dummy.gobj());
+
+	Glib::wrap_register (gtype, Widget_Vector::wrap_new);
+}
