@@ -30,12 +30,14 @@
 #	include <config.h>
 #endif
 
+#include <synfig/general.h>
+
 #include "layerduplicate.h"
 #include "layeradd.h"
 #include <synfig/context.h>
 #include <synfigapp/canvasinterface.h>
 
-#include <synfigapp/general.h>
+#include <synfigapp/localization.h>
 
 #endif
 
@@ -119,11 +121,11 @@ Action::LayerDuplicate::prepare()
 	if(!first_time())
 		return;
 
-	std::list<synfig::Layer::Handle>::const_iterator iter;
+	std::list<synfig::Layer::Handle>::const_iterator i;
 
-	for(iter=layers.begin();iter!=layers.end();++iter)
+	for(i=layers.begin();i!=layers.end();++i)
 	{
-		Layer::Handle layer(*iter);
+		Layer::Handle layer(*i);
 
 		Canvas::Handle subcanvas(layer->get_canvas());
 
@@ -181,7 +183,7 @@ Action::LayerDuplicate::export_dup_nodes(synfig::Layer::Handle layer, Canvas::Ha
 			{
 				canvas->find_value_node(name, true);
 			}
-			catch (Exception::IDNotFound x)
+			catch (const Exception::IDNotFound& x)
 			{
 				Action::Handle action(Action::create("ValueNodeAdd"));
 
